/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractComponentMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.IncomingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;

/**
 * The monitoring sub-function about incoming requests counters of the Petals BC
 * SOAP.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class IncomingHttpRequestsCount extends AbstractComponentMonitoringSubFunction {

    private final static String SUBFCT_NAME = "incoming-http-requests-count";

    private final static String SUBFCT_DESCRIPTION = "Counters of incoming HTTP requests";

    /**
     * The option to get statistics about incoming requests by their nature
     * (welcome servlet, service list servlet, unknown request, ws-request).
     */
    public static final String REQUEST_NATURE_SHORT_OPTION = "r";

    private static final Option REQUEST_NATURE_OPTION = Option.builder(REQUEST_NATURE_SHORT_OPTION).required(false)
            .desc("Get consolidate values by request natures.").build();

    /**
     * Header of the metric value associated to the information HTTP request
     * counter
     */
    protected final static String LABEL_INFORMATION = "Information";

    /**
     * Header of the metric value associated to the HTTP ws-request counter
     */
    protected final static String LABEL_WS = "WS";

    /**
     * Header of the metric value associated to the unknown HTTP request counter
     */
    protected final static String LABEL_UNKNOWN = "Unknown";

    /**
     * Header of the metric value associated to the service contracts HTTP request counter
     */
    protected final static String LABEL_SERVICE_CONTRACTS = "ServiceContracts";

    // TODO: Options declaration must be reviewed to match:
    // -n <component-name>
    // [-r]
    @Override
    protected void createOptions() {
        super.createOptions();
        getOptions().addOption(REQUEST_NATURE_OPTION);
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            if (cli.hasOption(REQUEST_NATURE_SHORT_OPTION)) {

                try {
                    final JMXClient client = getJMXClient();
                    final PetalsBcSoapMonitoringServiceClient bcSoapClient = client
                            .getPetalsBcSoapMonitoringServiceClient(componentIdentifier);

                    final StringBuilder reply = new StringBuilder();
                    final Long information = bcSoapClient.getInformationHttpRequestsCount();
                    if (information != null) {
                        reply.append(String.format("%s:%d ", LABEL_INFORMATION,
                                information.longValue()));
                    }
                    final Long unknown = bcSoapClient.getUnknownHttpRequestsCount();
                    if (unknown != null) {
                        reply.append(String.format("%s:%d ", LABEL_UNKNOWN, unknown.longValue()));
                    }
                    final Long ws = this.getConsolidatedValues(bcSoapClient, null, null,
                            null);
                    if (ws != null) {
                        reply.append(String.format("%s:%d ", LABEL_WS, ws.longValue()));
                    }
                    final Long serviceContracts = this.agregateServiceContractCounters(bcSoapClient);
                    if (serviceContracts != null) {
                        reply.append(String.format("%s:%d ", LABEL_SERVICE_CONTRACTS, serviceContracts.longValue()));
                    }

                    this.shell.getPrintStream().println(reply.toString());

                } catch (final ComponentMonitoringServiceDoesNotExistException e) {
                    throw new MonitoringSubFunctionException(this, String.format(
                            Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
                }

            } else {
                final List<String> missingOptions = new ArrayList<>();
                missingOptions.add(IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION);
                throw new MonitoringSubFunctionMissingOptionsException(this, missingOptions);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    /**
     * Get the consolidated value about WS request counters matching provided regexp
     * 
     * @param bcSoapClient
     *            The BC SOAP JMX client to use to retrieve counters
     * @param serviceRegexp
     *            The regexp of services for which the consolidation must be done
     * @param clientRegexp
     *            The regexp of client for which the consolidation must be done
     * @param execStatusRegexp
     *            Execution status filter. If <code>null</code>, no filter applied.
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     * @return The consolidated value matching provided regexp, or <code>null</code> if no values is available.
     */
    private Long getConsolidatedValues(final PetalsBcSoapMonitoringServiceClient bcSoapClient,
            final String serviceRegexp, final String clientRegexp, final String execStatusRegexp)
                    throws MonitoringSubFunctionException {

        try {

            final Pattern serviceNamePattern;
            try {
                serviceNamePattern = Pattern.compile(serviceRegexp == null ? ".*" : serviceRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format("Invalid regexp: '%s'", serviceRegexp), e);
            }

            final Pattern clientPattern;
            try {
                clientPattern = Pattern.compile(clientRegexp == null ? ".*" : clientRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format("Invalid regexp: '%s'", clientRegexp), e);
            }

            final Pattern execStatusPattern;
            try {
                execStatusPattern = Pattern.compile(execStatusRegexp == null ? ".*" : execStatusRegexp);
            } catch (final PatternSyntaxException e) {
                throw new MonitoringSubFunctionException(this, String.format("Invalid regexp: '%s'", execStatusRegexp),
                        e);
            }

            final Map<IncomingWsRequestKey, Long> values = bcSoapClient.getIncomingWsRequestsCount();
            if (values != null) {
                long consolidatedValue = 0;
                for (final Map.Entry<IncomingWsRequestKey, Long> entry : values.entrySet()) {

                    final IncomingWsRequestKey key = entry.getKey();
                    final String wsPath = key.getWsPath();
                    final Matcher serviceNameMatcher = serviceNamePattern.matcher(wsPath == null ? "" : wsPath);
                    final String clientIpAddress = key.getClientIpAddress();
                    final Matcher clientMatcher = clientPattern.matcher(clientIpAddress == null ? "" : clientIpAddress);
                    final String execStatus = key.getExecStatus().toString();
                    final Matcher execStatusMatcher = execStatusPattern.matcher(execStatus == null ? "" : execStatus);

                    if (serviceNameMatcher.matches() && clientMatcher.matches() && execStatusMatcher.matches()) {
                        consolidatedValue += entry.getValue().longValue();
                    }
                }

                return Long.valueOf(consolidatedValue);
            } else {
                return null;
            }

        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    /**
     * Get the sum of all service contracts calls.
     * 
     * @param bcSoapClient
     *            The BC SOAP JMX client to use to retrieve counters
     * @param serviceRegexp
     *            The regexp of services for which the consolidation must be done
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     * @return The sum of all service contracts calls, or <code>null</code> if no values is available.
     */
    private Long agregateServiceContractCounters(final PetalsBcSoapMonitoringServiceClient bcSoapClient)
            throws MonitoringSubFunctionException {

        try {
            final Map<String, Long> values = bcSoapClient.getServiceContractsHttpRequestsCount();
            if (values != null) {
                long consolidatedValue = 0;
                for (final Long value : values.values()) {
                    consolidatedValue += value.longValue();
                }
                return Long.valueOf(consolidatedValue);
            } else {
                return null;
            }

        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

}
