/**
 * Copyright (c) 2015-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import static org.ow2.petals.cli.extension.command.monitoring.mo.Constants.QUERY_RESULT_OUTPUT_FORMAT_S_D;
import static org.ow2.petals.cli.extension.command.monitoring.mo.Constants.QUERY_RESULT_OUTPUT_FORMAT_S_S;

import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractComponentMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;

/**
 * The monitoring sub-function about incoming service contract requests counters of the Petals BC SOAP.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class IncomingServiceContractRequestsCount extends AbstractComponentMonitoringSubFunction {

    private final static String SUBFCT_NAME = "incoming-service-contract-requests-count";

    private final static String SUBFCT_DESCRIPTION = "Counters of incoming web-service contract requests";

    /**
     * The option to get the list of web services
     */
    public static final String LIST_SERVICES_LONG_OPTION = "list-services";

    private static final Option LIST_SERVICES_OPTION = Option.builder().required(false).hasArg(false)
            .desc("List the web-service for which at least one request was received.")
            .longOpt(LIST_SERVICES_LONG_OPTION).build();

    /**
     * The option to query incoming service contract requests of web-service
     */
    public static final String QUERY_SERVICES_LONG_OPTION = "query-services";

    private static final Option QUERY_SERVICES_OPTION = Option.builder().required(false).optionalArg(true)
            .numberOfArgs(2).desc("Query about services having received service contract request")
            .longOpt(QUERY_SERVICES_LONG_OPTION).build();

    /**
     * The option to get the complete table of values
     */
    public static final String TABLE_SHORT_OPTION = "t";

    private static final Option TABLE_OPTION = Option.builder(TABLE_SHORT_OPTION).required(false)
            .desc("Get the complete table of values").build();

    /**
     * Query attribute to get index name
     */
    public static final String QUERY_ATTR_NAME = "NAME";

    /**
     * Query attribute to get the number of web-service contract requests.
     */
    public static final String QUERY_ATTR_COUNT = "COUNT";

    private final static String HEADER_SERVICE = "Service";

    private final static String HEADER_VALUE = "Value";

    // TODO: Options declaration must be reviewed to match:
    // -n <component-name>
    // [--list-operations] |
    // [--query-operations [NAME] | [COUNT <index>]] |
    // [-t]
    @Override
    protected void createOptions() {
        super.createOptions();
        getOptions().addOption(LIST_SERVICES_OPTION);
        getOptions().addOption(QUERY_SERVICES_OPTION);
        getOptions().addOption(TABLE_OPTION);
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException, MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException, MonitoringSubFunctionMissingOptionsException,
            MonitoringSubFunctionInvalidException, MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            // No extra option is accepted
            if (cli.getArgs().length > 0) {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            if (cli.hasOption(LIST_SERVICES_LONG_OPTION) && !cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listServices(componentIdentifier, false, null);

            } else if (!cli.hasOption(LIST_SERVICES_LONG_OPTION) && cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                final String[] queryArgs = cli.getOptionValues(QUERY_SERVICES_LONG_OPTION);
                if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null || queryArgs[0].isEmpty()) {
                    throw new MonitoringSubFunctionMissingArgumentException(this, QUERY_SERVICES_OPTION);
                } else if (!QUERY_ATTR_NAME.equals(queryArgs[0]) && !QUERY_ATTR_COUNT.equals(queryArgs[0])) {
                    throw new MonitoringSubFunctionInvalidArgumentException(this, QUERY_SERVICES_OPTION, queryArgs[0]);
                } else {
                    this.listServices(componentIdentifier, true, queryArgs);
                }

            } else if (!cli.hasOption(LIST_SERVICES_LONG_OPTION) && !cli.hasOption(QUERY_SERVICES_LONG_OPTION)
                    && cli.hasOption(TABLE_SHORT_OPTION)) {

                this.fullTable(componentIdentifier);

            } else {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        }
    }

    /**
     * List web-services for which a metric about web service contract requests is available
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listServices(final String componentIdentifier, final boolean hasQuery, final String[] queryArgs)
            throws MonitoringSubFunctionException {

        try {
            final JMXClient client = getJMXClient();
            final Map<String, Long> values = client.getPetalsBcSoapMonitoringServiceClient(componentIdentifier)
                    .getServiceContractsHttpRequestsCount();
            if (values != null) {
                if (hasQuery) {
                    this.query(values, queryArgs, QUERY_SERVICES_OPTION);
                } else {
                    for (final String serviceName : values.keySet()) {
                        this.shell.getPrintStream().println(serviceName);
                    }
                }
            }
        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this,
                    String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
        }
    }

    private void query(final Map<String, Long> values, final String[] queryArgs, final Option queryOption)
            throws MonitoringSubFunctionException {

        if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null || queryArgs[0].isEmpty()) {
            throw new MonitoringSubFunctionMissingArgumentException(this, queryOption);
        } else {
            final String index = queryArgs.length == 2 ? queryArgs[1] : null;
            final Map<String, Long> valuesByService;
            if (QUERY_ATTR_NAME.equals(queryArgs[0])) {
                for (final String serviceName : values.keySet()) {
                    this.shell.getPrintStream()
                            .println(String.format(QUERY_RESULT_OUTPUT_FORMAT_S_S, serviceName, serviceName));
                }
            } else {
                if (QUERY_ATTR_COUNT.equals(queryArgs[0])) {
                    valuesByService = values;
                } else {
                    throw new MonitoringSubFunctionInvalidArgumentException(this, queryOption, queryArgs[0]);
                }

                if (index != null) {
                    final Long value = valuesByService.get(index);
                    if (value == null) {
                        this.shell.getPrintStream().println(String.format("%d", 0));
                    } else {
                        this.shell.getPrintStream().println(String.format("%s", value));
                    }
                } else {
                    for (final Map.Entry<String, Long> entry : valuesByService.entrySet()) {
                        this.shell.getPrintStream().println(
                                String.format(QUERY_RESULT_OUTPUT_FORMAT_S_D, entry.getKey(), entry.getValue()));
                    }
                }
            }
        }
    }

    /**
     * Print the full table
     * 
     * @param componentIdentifier
     *            The identifier of the BC SOAP
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     */
    private void fullTable(final String componentIdentifier) throws MonitoringSubFunctionException {

        try {
            final JMXClient client = getJMXClient();
            final Map<String, Long> values = client.getPetalsBcSoapMonitoringServiceClient(componentIdentifier)
                    .getServiceContractsHttpRequestsCount();

            int maxSizeSvc = 0;
            int maxSizeValue = 0;
            if (values != null) {
                for (final Map.Entry<String, Long> entry : values.entrySet()) {
                    final String wsService = entry.getKey();
                    maxSizeSvc = Math.max(maxSizeSvc, wsService == null ? 0 : wsService.length());
                    maxSizeValue = Math.max(maxSizeValue, entry.getValue().toString().length());
                }
            }

            maxSizeSvc = Math.max(maxSizeSvc, HEADER_SERVICE.length());
            maxSizeValue = Math.max(maxSizeValue, HEADER_VALUE.length());

            final String rowHeaderFormat = "| %-" + maxSizeSvc + "s || %-" + maxSizeValue + "s |";
            final String header = String.format(rowHeaderFormat, HEADER_SERVICE, HEADER_VALUE);
            this.shell.getPrintStream().println(header);
            for (int i = 0; i < header.length(); i++) {
                this.shell.getPrintStream().append('-');
            }
            this.shell.getPrintStream().append('\n');

            if (values != null) {
                final String rowFormat = "| %-" + maxSizeSvc + "s || %" + maxSizeValue + "d |";
                for (final Map.Entry<String, Long> entry : values.entrySet()) {
                    final String wsService = entry.getKey();
                    this.shell.getPrintStream().println(
                            String.format(rowFormat, wsService == null ? "" : wsService, entry.getValue().longValue()));
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this,
                    String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
        }
    }
}
