/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractComponentMonitoringSubFunction;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.WsClientPoolKey;

/**
 * The monitoring sub-function about Ws-clients pools of the Petals BC SOAP.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class WsClientPools extends AbstractComponentMonitoringSubFunction {

    private final static String SUBFCT_NAME = "ws-clients-pools";

    private final static String SUBFCT_DESCRIPTION = "WS-clients pools statistics";

    /**
     * The option to get the list of external web-service operations invoked
     */
    public static final String LIST_OPERATIONS_LONG_OPTION = "list-operations";

    private static final Option LIST_OPERATIONS_OPTION = Option.builder().required(false).numberOfArgs(0).desc(
            "List the external web-service operations (expressed as 'operation'@'services' for which at least one request was sent.")
            .longOpt(LIST_OPERATIONS_LONG_OPTION).build();

    /**
     * The option to query external web-service operations invoked
     */
    public static final String QUERY_OPERATIONS_LONG_OPTION = "query-operations";

    private static final Option QUERY_OPERATIONS_OPTION = Option.builder().required(false).optionalArg(true)
            .numberOfArgs(2).desc("Query about external web-service operations").longOpt(QUERY_OPERATIONS_LONG_OPTION)
            .build();

    /**
     * The option to get the complete table of values
     */
    public static final String TABLE_SHORT_OPTION = "t";

    private static final Option TABLE_OPTION = Option.builder(TABLE_SHORT_OPTION).required(false)
            .desc("Get the complete table of values").build();

    /**
     * Query attribute to get index name
     */
    public static final String QUERY_ATTR_URL = "URL";

    /**
     * Query attribute to get the number of clients currently in use
     */
    public static final String QUERY_ATTR_INUSE_CUR = "INUSE_CUR";

    /**
     * Query attribute to get the max number of clients in use.
     */
    public static final String QUERY_ATTR_INUSE_MAX = "INUSE_MAX";

    /**
     * Query attribute to get the number of exhaustions.
     */
    public static final String QUERY_ATTR_EXHAUSTIONS = "EXHAUSTIONS";

    private final static String HEADER_SVC = "Service";

    private final static String HEADER_OPERATION = "Operation";

    private final static String HEADER_MEP = "MEP";

    private final static String HEADER_VALUE_IN_USE_CURRENT = "In-use current";

    private final static String HEADER_VALUE_IN_USE_MAX = "In-use max";

    private final static String HEADER_VALUE_EXHAUSTIONS = "Exhaustions";

    // TODO: Options declaration must be reviewed to match:
    // -n <component-name>
    // [--list-operations] |
    // [--query-operations [URL] | [INUSE_CUR|INUSE_MAX|EXHAUSTIONS <index>]] |
    // [-t]
    @Override
    protected void createOptions() {
        super.createOptions();
        getOptions().addOption(LIST_OPERATIONS_OPTION);
        getOptions().addOption(QUERY_OPERATIONS_OPTION);
        getOptions().addOption(TABLE_OPTION);
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            if (cli.hasOption(LIST_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(QUERY_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listOperations(componentIdentifier, false, null);

            } else if (!cli.hasOption(LIST_OPERATIONS_LONG_OPTION)
                    && cli.hasOption(QUERY_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listOperations(componentIdentifier, true,
                        cli.getOptionValues(QUERY_OPERATIONS_LONG_OPTION));

            } else if (!cli.hasOption(LIST_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(QUERY_OPERATIONS_LONG_OPTION)
                    && cli.hasOption(TABLE_SHORT_OPTION)) {

                this.fullTable(componentIdentifier);

            } else {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        }
    }

    /**
     * List operations for which a metric is available
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listOperations(final String componentIdentifier, final boolean hasQuery,
            final String[] queryArgs) throws MonitoringSubFunctionException {

        try {
            final JMXClient client = getJMXClient();
            final Map<WsClientPoolKey, Long> wsClientPoolClientsInUseCurrent = client
                    .getPetalsBcSoapMonitoringServiceClient(componentIdentifier)
                    .getWsClientPoolClientsInUseCurrent();
            final Map<WsClientPoolKey, Long> wsClientPoolClientsInUseMax = client
                    .getPetalsBcSoapMonitoringServiceClient(componentIdentifier)
                    .getWsClientPoolClientsInUseMax();
            final Map<WsClientPoolKey, Long> wsClientPoolExhaustions = client
                    .getPetalsBcSoapMonitoringServiceClient(componentIdentifier)
                    .getWsClientPoolExhaustions();

            if (hasQuery) {
                this.query(wsClientPoolClientsInUseCurrent, wsClientPoolClientsInUseMax,
                        wsClientPoolExhaustions, queryArgs, QUERY_OPERATIONS_OPTION);
            } else {
                final Set<WsClientPoolKey> entries = WsClientPools.mergeKeys(
                        wsClientPoolClientsInUseCurrent, wsClientPoolClientsInUseMax,
                        wsClientPoolExhaustions);

                final List<String> printedOperation = new ArrayList<>();
                for (final WsClientPoolKey entry : entries) {
                    final String operation = WsClientPools.getOperationName(entry);
                    if (!printedOperation.contains(operation)) {
                        this.shell.getPrintStream().println(operation);
                        printedOperation.add(operation);
                    }
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    private static String getOperationName(final WsClientPoolKey entry) {
        return entry.getOperation() + "@" + entry.getService();
    }

    private void query(final Map<WsClientPoolKey, Long> wsClientPoolClientsInUseCurrent,
            final Map<WsClientPoolKey, Long> wsClientPoolClientsInUseMax,
            final Map<WsClientPoolKey, Long> wsClientPoolExhaustions, final String[] queryArgs,
            final Option queryOption)
            throws MonitoringSubFunctionException {

        if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null
                || queryArgs[0].isEmpty()) {
            throw new MonitoringSubFunctionMissingArgumentException(this, queryOption);
        } else {
            final String index = queryArgs.length == 2 ? queryArgs[1] : null;
            final Set<WsClientPoolKey> entries = WsClientPools.mergeKeys(
                    wsClientPoolClientsInUseCurrent, wsClientPoolClientsInUseMax,
                    wsClientPoolExhaustions);

            if (QUERY_ATTR_URL.equals(queryArgs[0])) {

                final List<String> printedService = new ArrayList<>();
                for (final WsClientPoolKey entry : entries) {
                    final String operation = WsClientPools.getOperationName(entry);
                    if (!printedService.contains(operation)) {
                        this.shell.getPrintStream().println(
                                String.format("%s!%s", operation, operation));
                        printedService.add(operation);
                    }
                }
            } else {

                final Map<String, Long> valuesByOperation;
                if (QUERY_ATTR_INUSE_CUR.equals(queryArgs[0])) {
                    valuesByOperation = this.agregate(wsClientPoolClientsInUseCurrent);
                } else if (QUERY_ATTR_INUSE_MAX.equals(queryArgs[0])) {
                    valuesByOperation = this.agregate(wsClientPoolClientsInUseMax);
                } else if (QUERY_ATTR_EXHAUSTIONS.equals(queryArgs[0])) {
                    valuesByOperation = this.agregate(wsClientPoolExhaustions);
                } else {
                    throw new MonitoringSubFunctionInvalidArgumentException(this, queryOption,
                            queryArgs[0]);
                }

                if (index != null) {
                    final Long value = valuesByOperation.get(index);
                    if (value == null) {
                        this.shell.getPrintStream().println(String.format("%d", 0));
                    } else {
                        this.shell.getPrintStream().println(String.format("%d", value.longValue()));
                    }
                } else {
                    final List<String> printedOperations = new ArrayList<>();
                    // We loop over all entries to get null value (for example
                    // if no exhaustion occurs on an operation, it will not
                    // appear in the aggregated map.
                    for (final WsClientPoolKey entry : entries) {
                        final String operationName = WsClientPools.getOperationName(entry);
                        if (!printedOperations.contains(operationName)) {
                            final Long value = valuesByOperation.get(operationName);
                            this.shell.getPrintStream().println(
                                    String.format("%s!%d", operationName, value == null ? 0l
                                            : value.longValue()));
                            printedOperations.add(operationName);
                        }
                    }
                }
            }
        }
    }

    /**
     * Aggregates values and returns the value for each service/operation
     * (expressed as <code>operation@service</code>)
     * 
     * @param values
     *            The values to aggregate
     * @return The aggregated value for each service/operation (expressed as
     *         <code>operation@service</code>)
     */
    private Map<String, Long> agregate(final Map<WsClientPoolKey, Long> values) {
        final Map<String, Long> results = new HashMap<>();
        for (final Map.Entry<WsClientPoolKey, Long> entry : values.entrySet()) {
            final WsClientPoolKey key = entry.getKey();
            final String operation = WsClientPools.getOperationName(key);
            if (results.containsKey(operation)) {
                results.put(operation, results.get(operation) + entry.getValue());
            } else {
                results.put(operation, entry.getValue());
            }
        }
        return results;
    }

    /**
     * Print the full table
     * 
     * @param componentIdentifier
     *            The identifier of the BC SOAP
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     */
    private void fullTable(final String componentIdentifier) throws MonitoringSubFunctionException {

        try {
            final JMXClient client = getJMXClient();
            final Map<WsClientPoolKey, Long> wsClientPoolClientsInUseCurrent = client
                    .getPetalsBcSoapMonitoringServiceClient(
                    componentIdentifier).getWsClientPoolClientsInUseCurrent();
            final Map<WsClientPoolKey, Long> wsClientPoolClientsInUseMax = client
                    .getPetalsBcSoapMonitoringServiceClient(
                    componentIdentifier).getWsClientPoolClientsInUseMax();
            final Map<WsClientPoolKey, Long> wsClientPoolExhaustions = client
                    .getPetalsBcSoapMonitoringServiceClient(
                    componentIdentifier).getWsClientPoolExhaustions();
            
            final Set<WsClientPoolKey> entries = WsClientPools.mergeKeys(
                    wsClientPoolClientsInUseCurrent,
                    wsClientPoolClientsInUseMax, wsClientPoolExhaustions);

            int maxSizeSvc = 0;
            int maxSizeOperation = 0;
            int maxSizeMep = 0;
            for (final WsClientPoolKey entry : entries) {
                final String service = entry.getService();
                maxSizeSvc = Math.max(maxSizeSvc, service == null ? 0 : service.length());
                final String operation = entry.getOperation();
                maxSizeOperation = Math.max(maxSizeOperation,
                        operation == null ? 0 : operation.length());
                final String mep = entry.getMep();
                maxSizeMep = Math.max(maxSizeMep, mep == null ? 0 : mep.length());
            }

            maxSizeSvc = Math.max(maxSizeSvc, HEADER_SVC.length());
            maxSizeOperation = Math.max(maxSizeOperation, HEADER_OPERATION.length());
            maxSizeMep = Math.max(maxSizeMep, HEADER_MEP.length());
            final int maxSizeValue = Math.max(Math.max(HEADER_VALUE_IN_USE_MAX.length(),
                    HEADER_VALUE_IN_USE_CURRENT.length()), HEADER_VALUE_EXHAUSTIONS.length());

            final String rowHeaderFormat = "| %-" + maxSizeSvc + "s | %-" + maxSizeOperation
                    + "s | %-" + maxSizeMep + "s || %-" + maxSizeValue + "s | %-" + maxSizeValue + "s | %-" + maxSizeValue + "s |";
            final String header = String.format(rowHeaderFormat, HEADER_SVC,
                    HEADER_OPERATION, HEADER_MEP, HEADER_VALUE_IN_USE_CURRENT, HEADER_VALUE_IN_USE_MAX, HEADER_VALUE_EXHAUSTIONS);
            this.shell.getPrintStream().println(header);
            for (int i = 0; i < header.length(); i++) {
                this.shell.getPrintStream().append('-');
            }
            this.shell.getPrintStream().append('\n');

            final String rowFormat = "| %-" + maxSizeSvc + "s | %-" + maxSizeOperation + "s | %-"
                    + maxSizeMep + "s || %" + maxSizeValue + "d | %" + maxSizeValue + "d | %"
                    + maxSizeValue + "d |";
            for (final WsClientPoolKey entry : entries) {

                final String service = entry.getService();
                final String operation = entry.getOperation();
                final String mep = entry.getMep();

                final Long valueInUseCurrent = wsClientPoolClientsInUseCurrent.get(entry);
                final Long valueInUseMax = wsClientPoolClientsInUseMax.get(entry);
                final Long valueExhaustions = wsClientPoolExhaustions.get(entry);

                this.shell.getPrintStream().println(
                        String.format(rowFormat, service == null ? "" : service,
                                operation == null ? "" : operation, mep == null ? "" : mep,
                                valueInUseCurrent == null ? 0l : valueInUseCurrent,
                                valueInUseMax == null ? 0l : valueInUseMax,
                                valueExhaustions == null ? 0l : valueExhaustions));
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    /**
     * Creates a list of all keys contained in all ws-clients pools value map.
     * 
     * @param wsClientPoolClientsInUseCurrent
     * @param wsClientPoolClientsInUseMax
     * @param wsClientPoolExhaustions
     * @return The list of all keys
     */
    private static Set<WsClientPoolKey> mergeKeys(
            final Map<WsClientPoolKey, Long> wsClientPoolClientsInUseCurrent,
            final Map<WsClientPoolKey, Long> wsClientPoolClientsInUseMax,
            final Map<WsClientPoolKey, Long> wsClientPoolExhaustions) {
        
        final Set<WsClientPoolKey> mergedKeys = new HashSet<>();

        mergedKeys.addAll(wsClientPoolClientsInUseCurrent.keySet());
        mergedKeys.addAll(wsClientPoolClientsInUseMax.keySet());
        mergedKeys.addAll(wsClientPoolExhaustions.keySet());
        
        return mergedKeys;
    }
}
