/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractComponentMonitoringSubFunction;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;

/**
 * The monitoring sub-function about the message exchange acceptor pool of a component.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MexAcceptorPool extends AbstractComponentMonitoringSubFunction implements MonitoringSubFunction {

    private final static String SUBFCT_NAME = "mex-acceptor-thread-pool";

    private final static String SUBFCT_DESCRIPTION = "Message exchange acceptor pool statistics";

    /**
     * Header of the metric value associated to the max active threads of the acceptor pool
     */
    public static final String HEADER_ACTIVE_MAX = "ActiveThreadsMax";

    /**
     * Header of the metric value associated to the current active threads of the acceptor pool
     */
    public static final String HEADER_ACTIVE_CURRENT = "ActiveThreadsCurrent";

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException, MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException, MonitoringSubFunctionMissingOptionsException,
            MonitoringSubFunctionInvalidException, MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            final JMXClient client = getJMXClient();
            try {

                final ComponentMonitoringServiceClient monitoringService = client
                        .getComponentMonitoringServiceClient(componentIdentifier);

                final StringBuilder reply = new StringBuilder();
                final Integer activeThreadsMax = monitoringService.getMexAcceptorThreadPoolActiveThreadsMaxSize();
                if (activeThreadsMax != null) {
                    reply.append(String.format("%s:%d ", HEADER_ACTIVE_MAX, activeThreadsMax.longValue()));
                }

                final Integer activeThreadsCurrent = monitoringService
                        .getMexAcceptorThreadPoolActiveThreadsCurrentSize();
                if (activeThreadsCurrent != null) {
                    reply.append(String.format("%s:%d ", HEADER_ACTIVE_CURRENT, activeThreadsCurrent.longValue()));
                }

                this.shell.getPrintStream().println(reply.toString());

            } catch (final ComponentMonitoringServiceDoesNotExistException e) {
                throw new MonitoringSubFunctionException(this,
                        String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

}
