/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.se.flowable;

import static org.ow2.petals.cli.extension.command.monitoring.mo.Constants.QUERY_RESULT_OUTPUT_FORMAT_S_D;
import static org.ow2.petals.cli.extension.command.monitoring.mo.Constants.QUERY_RESULT_OUTPUT_FORMAT_S_S;
import static org.ow2.petals.se.flowable.clientserver.api.monitoring.MonitoringService.PROCESS_DEFINITION_INFO_ID_ACTIVE_INSTANCES_COUNTER;
import static org.ow2.petals.se.flowable.clientserver.api.monitoring.MonitoringService.PROCESS_DEFINITION_INFO_ID_ENDED_INSTANCES_COUNTER;
import static org.ow2.petals.se.flowable.clientserver.api.monitoring.MonitoringService.PROCESS_DEFINITION_INFO_ID_SUSPENDED_INSTANCES_COUNTER;
import static org.ow2.petals.se.flowable.clientserver.api.monitoring.MonitoringService.PROCESS_DEFINITION_INFO_ID_SUSPENSION_STATE;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractComponentMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.se.flowable.SeFlowableMonitoringServiceClient;

/**
 * The monitoring sub-function about process definition counters of the Petals BC SOAP.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ProcessDefinitionsCount extends AbstractComponentMonitoringSubFunction {

    private final static String SUBFCT_NAME = "process-definitions-count";

    private final static String SUBFCT_DESCRIPTION = "Counters of process definitions";

    /**
     * The option to get the list of process definitions
     */
    public static final String LIST_PROCESS_DEFINITIONS_LONG_OPTION = "list-process-definitions";

    private static final Option LIST_PROCESS_DEFINITIONS_OPTION = Option.builder().required(false).hasArg(false)
            .desc("List the process definitions deployed.").longOpt(LIST_PROCESS_DEFINITIONS_LONG_OPTION).build();

    /**
     * The option to query process definitions
     */
    public static final String QUERY_PROCESS_DEFINITIONS_LONG_OPTION = "query-process-definitions";

    private static final Option QUERY_PROCESS_DEFINITIONS_OPTION = Option.builder().required(false).optionalArg(true)
            .numberOfArgs(2).desc("Query about process definitions").longOpt(QUERY_PROCESS_DEFINITIONS_LONG_OPTION)
            .build();

    /**
     * The option to get the complete table of values
     */
    public static final String TABLE_SHORT_OPTION = "t";

    private static final Option TABLE_OPTION = Option.builder(TABLE_SHORT_OPTION).required(false)
            .desc("Get the complete table of values").build();

    /**
     * Query attribute to get index name
     */
    public static final String QUERY_ATTR_NAME = "NAME";

    /**
     * Query attribute to get the number of active process instances of the process definition.
     */
    public static final String QUERY_ATTR_ACTIVE = "ACTIVE";

    /**
     * Query attribute to get the number of suspended process instances of the process definition.
     */
    public static final String QUERY_ATTR_SUSPENDED = "SUSPENDED";

    /**
     * Query attribute to get the number of ended process instances of the process definition processed.
     */
    public static final String QUERY_ATTR_ENDED = "ENDED";

    private final static String HEADER_PROCESS_DEFINITION = "Process definition";

    private final static String HEADER_PROCESS_DEFINITION_STATE = "State";

    private final static String HEADER_ACTIVE_COUNTER = "Active";

    private final static String HEADER_SUSPENDED_COUNTER = "Suspended";

    private final static String HEADER_ENDED_COUNTER = "Ended";

    // TODO: Options declaration must be reviewed to match:
    // -n <component-name>
    // [--list-process-definition] |
    // [--query-process-definitions [NAME] | [ACTIVE|SUSPENDED|ENDED <index>]] |
    // [-t]
    @Override
    protected void createOptions() {
        super.createOptions();
        getOptions().addOption(LIST_PROCESS_DEFINITIONS_OPTION);
        getOptions().addOption(QUERY_PROCESS_DEFINITIONS_OPTION);
        getOptions().addOption(TABLE_OPTION);
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException, MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException, MonitoringSubFunctionMissingOptionsException,
            MonitoringSubFunctionInvalidException, MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            // No extra option is accepted
            if (cli.getArgs().length > 0) {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            if (cli.hasOption(LIST_PROCESS_DEFINITIONS_LONG_OPTION)
                    && !cli.hasOption(QUERY_PROCESS_DEFINITIONS_LONG_OPTION) && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listProcessDefinitions(componentIdentifier, false, null);

            } else if (!cli.hasOption(LIST_PROCESS_DEFINITIONS_LONG_OPTION)
                    && cli.hasOption(QUERY_PROCESS_DEFINITIONS_LONG_OPTION) && !cli.hasOption(TABLE_SHORT_OPTION)) {

                final String[] queryArgs = cli.getOptionValues(QUERY_PROCESS_DEFINITIONS_LONG_OPTION);
                if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null || queryArgs[0].isEmpty()) {
                    throw new MonitoringSubFunctionMissingArgumentException(this, QUERY_PROCESS_DEFINITIONS_OPTION);
                } else if (!QUERY_ATTR_NAME.equals(queryArgs[0]) && !QUERY_ATTR_ACTIVE.equals(queryArgs[0])
                        && !QUERY_ATTR_SUSPENDED.equals(queryArgs[0]) && !QUERY_ATTR_ENDED.equals(queryArgs[0])) {
                    throw new MonitoringSubFunctionInvalidArgumentException(this, QUERY_PROCESS_DEFINITIONS_OPTION,
                            queryArgs[0]);
                } else {
                    this.listProcessDefinitions(componentIdentifier, true,
                            cli.getOptionValues(QUERY_PROCESS_DEFINITIONS_LONG_OPTION));
                }

            } else if (!cli.hasOption(LIST_PROCESS_DEFINITIONS_LONG_OPTION)
                    && !cli.hasOption(QUERY_PROCESS_DEFINITIONS_LONG_OPTION) && cli.hasOption(TABLE_SHORT_OPTION)) {

                this.fullTable(componentIdentifier);

            } else {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        }
    }

    /**
     * List process definitions
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listProcessDefinitions(final String componentIdentifier, final boolean hasQuery,
            final String[] queryArgs) throws MonitoringSubFunctionException {

        try {
            final JMXClient client = getJMXClient();
            final Map<String, Long[]> values = client.getSeFlowableMonitoringServiceClient(componentIdentifier)
                    .getProcessDefinitions();
            if (values != null) {
                if (hasQuery) {
                    this.query(values, queryArgs, QUERY_PROCESS_DEFINITIONS_OPTION);
                } else {
                    for (final String processDefinition : values.keySet()) {
                        this.shell.getPrintStream().println(processDefinition);
                    }
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this,
                    String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
        }
    }

    private void query(final Map<String, Long[]> values, final String[] queryArgs, final Option queryOption)
            throws MonitoringSubFunctionException {

        if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null || queryArgs[0].isEmpty()) {
            throw new MonitoringSubFunctionMissingArgumentException(this, queryOption);
        } else {
            final String index = queryArgs.length == 2 ? queryArgs[1] : null;
            if (QUERY_ATTR_NAME.equals(queryArgs[0])) {
                for (final String processDefinition : values.keySet()) {
                    this.shell.getPrintStream().println(
                            String.format(QUERY_RESULT_OUTPUT_FORMAT_S_S, processDefinition, processDefinition));
                }
            } else {
                final Map<String, Long> valuesWithoutProcessDefStatus;
                if (QUERY_ATTR_ACTIVE.equals(queryArgs[0])) {
                    valuesWithoutProcessDefStatus = this.getCounterValues(values,
                            PROCESS_DEFINITION_INFO_ID_ACTIVE_INSTANCES_COUNTER);
                } else if (QUERY_ATTR_SUSPENDED.equals(queryArgs[0])) {
                    valuesWithoutProcessDefStatus = this.getCounterValues(values,
                            PROCESS_DEFINITION_INFO_ID_SUSPENDED_INSTANCES_COUNTER);
                } else if (QUERY_ATTR_ENDED.equals(queryArgs[0])) {
                    valuesWithoutProcessDefStatus = this.getCounterValues(values,
                            PROCESS_DEFINITION_INFO_ID_ENDED_INSTANCES_COUNTER);
                } else {
                    throw new MonitoringSubFunctionInvalidArgumentException(this, queryOption, queryArgs[0]);
                }

                if (index != null) {
                    final Long value = valuesWithoutProcessDefStatus.get(index);
                    if (value == null) {
                        this.shell.getPrintStream().println(String.format("%d", 0));
                    } else {
                        this.shell.getPrintStream().println(String.format("%s", value));
                    }
                } else {
                    for (final Map.Entry<String, Long> entry : valuesWithoutProcessDefStatus.entrySet()) {
                        this.shell.getPrintStream().println(
                                String.format(QUERY_RESULT_OUTPUT_FORMAT_S_D, entry.getKey(), entry.getValue()));
                    }
                }
            }
        }
    }

    /**
     * Values of the counter identified by the given {@code counterId}.
     * 
     * @param values
     *            Raw values coming from {@link SeFlowableMonitoringServiceClient#getProcessDefinitions()}
     * @param counterId
     *            The counter for which we want values
     * @return Counter value for each process definition
     */
    private Map<String, Long> getCounterValues(final Map<String, Long[]> values, final int counterId) {
        final Map<String, Long> results = new HashMap<>();
        for (final Entry<String, Long[]> entry : values.entrySet()) {
            results.put(entry.getKey(), entry.getValue()[counterId]);
        }
        return results;
    }

    /**
     * Print the full table
     * 
     * @param componentIdentifier
     *            The identifier of the BC SOAP
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     */
    private void fullTable(final String componentIdentifier) throws MonitoringSubFunctionException {

        try {
            final JMXClient client = getJMXClient();
            final Map<String, Long[]> values = client.getSeFlowableMonitoringServiceClient(componentIdentifier)
                    .getProcessDefinitions();

            int maxSizeProcDef = 0;
            int maxSizeProcDefState = 0;
            int maxSizeActive = 0;
            int maxSizeSuspended = 0;
            int maxSizeEnded = 0;
            if (values != null) {
                for (final Entry<String, Long[]> entry : values.entrySet()) {
                    final String key = entry.getKey();
                    final Long[] keyValue = entry.getValue();
                    maxSizeProcDef = Math.max(maxSizeProcDef, key.length());
                    maxSizeProcDefState = Math.max(maxSizeProcDefState,
                            keyValue[PROCESS_DEFINITION_INFO_ID_SUSPENSION_STATE].toString().length());
                    maxSizeActive = Math.max(maxSizeActive,
                            keyValue[PROCESS_DEFINITION_INFO_ID_ACTIVE_INSTANCES_COUNTER].toString().length());
                    maxSizeSuspended = Math.max(maxSizeSuspended,
                            keyValue[PROCESS_DEFINITION_INFO_ID_SUSPENDED_INSTANCES_COUNTER].toString().length());
                    maxSizeEnded = Math.max(maxSizeEnded,
                            keyValue[PROCESS_DEFINITION_INFO_ID_ENDED_INSTANCES_COUNTER].toString().length());
                }
            }

            maxSizeProcDef = Math.max(maxSizeProcDef, HEADER_PROCESS_DEFINITION.length());
            maxSizeProcDefState = Math.max(maxSizeProcDefState, HEADER_PROCESS_DEFINITION_STATE.length());
            maxSizeActive = Math.max(maxSizeActive, HEADER_ACTIVE_COUNTER.length());
            maxSizeSuspended = Math.max(maxSizeSuspended, HEADER_SUSPENDED_COUNTER.length());
            maxSizeEnded = Math.max(maxSizeEnded, HEADER_ENDED_COUNTER.length());

            final String rowHeaderFormat = "| %-" + maxSizeProcDef + "s || %-" + maxSizeProcDefState + "s | %-"
                    + maxSizeActive + "s | %-" + maxSizeSuspended + "s | %-" + maxSizeEnded + "s |";
            final String header = String.format(rowHeaderFormat, HEADER_PROCESS_DEFINITION,
                    HEADER_PROCESS_DEFINITION_STATE, HEADER_ACTIVE_COUNTER, HEADER_SUSPENDED_COUNTER,
                    HEADER_ENDED_COUNTER);
            this.shell.getPrintStream().println(header);
            for (int i = 0; i < header.length(); i++) {
                this.shell.getPrintStream().append('-');
            }
            this.shell.getPrintStream().append('\n');

            if (values != null) {
                final String rowFormat = "| %-" + maxSizeProcDef + "s || %-" + maxSizeProcDefState + "s | %-"
                        + maxSizeActive + "s | %-" + maxSizeSuspended + "s | %" + maxSizeEnded + "d |";
                for (final Entry<String, Long[]> entry : values.entrySet()) {

                    final Long[] entryValue = entry.getValue();

                    this.shell.getPrintStream()
                            .println(String.format(rowFormat, entry.getKey(),
                                    entryValue[PROCESS_DEFINITION_INFO_ID_SUSPENSION_STATE],
                                    entryValue[PROCESS_DEFINITION_INFO_ID_ACTIVE_INSTANCES_COUNTER],
                                    entryValue[PROCESS_DEFINITION_INFO_ID_SUSPENDED_INSTANCES_COUNTER],
                                    entryValue[PROCESS_DEFINITION_INFO_ID_ENDED_INSTANCES_COUNTER]));
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this,
                    String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
        }
    }
}
