/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.local;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.exception.LocalTransporterMonitoringServiceErrorException;

/**
 * The monitoring sub-function about delivered messages by the local transporter
 * of the Petals container
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DeliveredMessages extends AbstractMonitoringSubFunction implements MonitoringSubFunction {

    private final static String SUBFCT_NAME = "delivered-messages";

    private final static String SUBFCT_DESCRIPTION = "Delivered messages";

    /**
     * Header of the metric value associated to the delivered message in state 'PENDING'
     */
    public static final String HEADER_PENDING = "Pending";

    /**
     * Header of the metric value associated to the delivered message in state 'SUCCEEDED'
     */
    public static final String HEADER_SUCCEEDED = "Succeeded";

    /**
     * Header of the metric value associated to the delivered message in state 'ERROR'
     */
    public static final String HEADER_ERROR = "Error";

    /**
     * The sub-function options
     */
    private Options options;

    /**
     * The shell in which the sub-function is running
     */
    private Shell shell;

    public DeliveredMessages() {
        super();
        this.createOptions();
    }

    private void createOptions() {
        this.options = new Options();
    }

    @Override
    public Options getOptions() {
        return this.options;
    }

    @Override
    public void resetOptions() {
        this.createOptions();
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void setShell(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public Shell getShell() {
        return this.shell;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            final org.ow2.petals.jmx.api.api.monitoring.container.transporter.local.DeliveredMessages deliveredMessages = getJMXClient()
                        .getLocalTransporterMonitoringServiceClient()
                        .getDeliveredMessage();

            this.shell.getPrintStream()
                    .println(String.format("%s:%d %s:%d %s:%d", HEADER_PENDING, deliveredMessages.getPendingCounter(),
                            HEADER_SUCCEEDED, deliveredMessages.getSucceededCounter(), HEADER_ERROR,
                            deliveredMessages.getErrorCounter()));

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final LocalTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }
}
