/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.remote;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.OutgoingConnectionObjectPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceErrorException;

/**
 * The monitoring sub-function about outgoing connections pool of the TCP transporter.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class OutgoingConnections extends AbstractMonitoringSubFunction implements MonitoringSubFunction {

    private final static String SUBFCT_NAME = "outgoing-connections";

    private final static String SUBFCT_DESCRIPTION = "Outgoing connection statistics";

    /**
     * The option to get the list of containers
     */
    public static final String LIST_CONTAINERS_LONG_OPTION = "list-containers";

    private static final Option LIST_CONTAINERS_OPTION = Option.builder().required(false).numberOfArgs(0)
            .desc("List the remote containers which at connections can be established.")
            .longOpt(LIST_CONTAINERS_LONG_OPTION).build();

    /**
     * The option to query delivered messages of containers
     */
    public static final String QUERY_CONTAINERS_LONG_OPTION = "query-containers";

    private static final Option QUERY_CONTAINERS_OPTION = Option.builder().required(false).optionalArg(true)
            .numberOfArgs(2).desc("Query about remote containers").longOpt(QUERY_CONTAINERS_LONG_OPTION).build();

    /**
     * The option to get the complete table of values
     */
    public static final String TABLE_SHORT_OPTION = "t";

    private static final Option TABLE_OPTION = Option.builder(TABLE_SHORT_OPTION).required(false)
            .desc("Get the complete table of values").build();

    /**
     * Query attribute to get index name
     */
    public static final String QUERY_ATTR_NAME = "NAME";

    /**
     * Query attribute to get the max number of active connections, required the index on which the query must be run
     */
    public static final String QUERY_ATTR_ACTIVE_CONNECTIONS_MAX = "MAX_ACTIVE";

    /**
     * Query attribute to get the current number of active connections, required the index on which the query must be
     * run
     */
    public static final String QUERY_ATTR_ACTIVE_CONNECTIONS = "ACTIVE";

    /**
     * Query attribute to get the max number of idle connections, required the index on which the query must be run
     */
    public static final String QUERY_ATTR_IDLE_CONNECTIONS_MAX = "MAX_IDLE";

    /**
     * Query attribute to get the current number of idle connections, required the index on which the query must be run
     */
    public static final String QUERY_ATTR_IDLE_CONNECTIONS = "IDLE";

    /**
     * Query attribute to get the max size of the connection pool, required the index on which the query must be run
     */
    public static final String QUERY_ATTR_MAX_SIZE = "MAX_SIZE";

    /**
     * Query attribute to get the min size of the connection pool, required the index on which the query must be run
     */
    public static final String QUERY_ATTR_MIN_SIZE = "MIN_SIZE";

    /**
     * Query attribute to get the number of connection established since the last start of the remote transporter,
     * required the index on which the query must be run
     */
    public static final String QUERY_ATTR_ESTABLISHED_CONNECTIONS = "ESTABLISHED";

    /**
     * Header of the metric value associated to the remote-container
     */
    public static final String HEADER_REMOTE_CONTAINER = "RemoteContainer";

    /**
     * Header of the metric value associated to the max active connections of the connection pool
     */
    public static final String HEADER_ACTIVE_CONNECTION_MAX = "ActiveConnectionsMax";

    /**
     * Header of the metric value associated to the current active connections of the connection pool
     */
    public static final String HEADER_ACTIVE_CONNECTION_CURRENT = "ActiveConnectionsCurrent";

    /**
     * Header of the metric value associated to the max idle connections of the connection pool
     */
    public static final String HEADER_IDLE_CONNECTION_MAX = "IdleConnectionsMax";

    /**
     * Header of the metric value associated to the current idle connections of the connection pool
     */
    public static final String HEADER_IDLE_CONNECTION_CURRENT = "IdleConnectionsCurrent";

    /**
     * Header of the metric value associated to the max size of the connection pool
     */
    public static final String HEADER_MAX_SIZE = "MaxSize";

    /**
     * Header of the metric value associated to the min size of the connection pool
     */
    public static final String HEADER_MIN_SIZE = "MinSize";

    /**
     * Header of the metric value associated to the number of established connections
     */
    public static final String HEADER_ESTABLISHED_CONNECTIONS = "EstablishedConnections";

    /**
     * The sub-function options
     */
    private Options options;

    /**
     * The shell in which the sub-function is running
     */
    private Shell shell;

    public OutgoingConnections() {
        super();
        this.createOptions();
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    // TODO: Options declaration must be reviewed to match:
    // [--list-containers] |
    // [--query-containers [NAME] | [MAX_ACTIVE|ACTIVE|MAX_IDLE|IDLE|MAX_SIZE|MIN_SIZE|ESTABLISHED <index>]] |
    // [-t]
    private void createOptions() {
        this.options = new Options();
        this.options.addOption(LIST_CONTAINERS_OPTION);
        this.options.addOption(QUERY_CONTAINERS_OPTION);
        this.options.addOption(TABLE_OPTION);
    }

    @Override
    public Options getOptions() {
        return this.options;
    }

    @Override
    public void resetOptions() {
        this.createOptions();
    }

    @Override
    public void setShell(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public Shell getShell() {
        return this.shell;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException, MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException, MonitoringSubFunctionMissingOptionsException,
            MonitoringSubFunctionInvalidException, MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args, true);

            if (cli.hasOption(LIST_CONTAINERS_LONG_OPTION) && !cli.hasOption(QUERY_CONTAINERS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listContainers(false, null);

            } else if (!cli.hasOption(LIST_CONTAINERS_LONG_OPTION) && cli.hasOption(QUERY_CONTAINERS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listContainers(true, cli.getOptionValues(QUERY_CONTAINERS_LONG_OPTION));

            } else if (!cli.hasOption(LIST_CONTAINERS_LONG_OPTION) && !cli.hasOption(QUERY_CONTAINERS_LONG_OPTION)
                    && cli.hasOption(TABLE_SHORT_OPTION)) {

                this.fullTable();

            } else {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        }
    }

    /**
     * List containers for which a metric 'connection pool' is available
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listContainers(final boolean hasQuery, final String[] queryArgs)
            throws MonitoringSubFunctionException {

        try {
            final Map<String, OutgoingConnectionObjectPoolMetrics> values = getJMXClient()
                    .getTcpTransporterMonitoringServiceClient().getOutgoingConnectionPoolMetrics();
            if (hasQuery) {
                this.query(values, queryArgs, QUERY_CONTAINERS_OPTION);
            } else {
                for (final String container : values.keySet()) {
                    this.shell.getPrintStream().println(container);
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final TcpTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final TcpTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    private void query(final Map<String, OutgoingConnectionObjectPoolMetrics> values, final String[] queryArgs,
            final Option queryOption) throws MonitoringSubFunctionException {

        if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null || queryArgs[0].isEmpty()) {
            throw new MonitoringSubFunctionMissingArgumentException(this, queryOption);
        } else {
            // No other arguments is accepted has option value
            for (final String optionValue : queryArgs) {
                if (optionValue.startsWith("-")) {
                    // Another argument is given as option value
                    throw new MonitoringSubFunctionBadArgumentNumberException(this);
                }
            }

            if (QUERY_ATTR_NAME.equals(queryArgs[0])) {
                for (final String container : values.keySet()) {
                    this.shell.getPrintStream().println(String.format("%s!%s", container, container));
                }
            } else {
                final Map<String, Long> valuesByQueryAttr = new HashMap<>();
                for (final Entry<String, OutgoingConnectionObjectPoolMetrics> entry : values.entrySet()) {
                    final Long queryAttrValue;
                    if (QUERY_ATTR_ACTIVE_CONNECTIONS_MAX.equals(queryArgs[0])) {
                        queryAttrValue = entry.getValue().getBorrowedObjectMaxNumber();
                    } else if (QUERY_ATTR_ACTIVE_CONNECTIONS.equals(queryArgs[0])) {
                        queryAttrValue = entry.getValue().getBorrowedObjectNumber();
                    } else if (QUERY_ATTR_IDLE_CONNECTIONS_MAX.equals(queryArgs[0])) {
                        queryAttrValue = entry.getValue().getIdleObjectMaxNumber();
                    } else if (QUERY_ATTR_IDLE_CONNECTIONS.equals(queryArgs[0])) {
                        queryAttrValue = entry.getValue().getIdleObjectNumber();
                    } else if (QUERY_ATTR_MAX_SIZE.equals(queryArgs[0])) {
                        queryAttrValue = entry.getValue().getMaxSize();
                    } else if (QUERY_ATTR_MIN_SIZE.equals(queryArgs[0])) {
                        queryAttrValue = entry.getValue().getMinIdleSize();
                    } else if (QUERY_ATTR_ESTABLISHED_CONNECTIONS.equals(queryArgs[0])) {
                        queryAttrValue = entry.getValue().getEstablished();
                    } else {
                        throw new MonitoringSubFunctionInvalidArgumentException(this, queryOption, queryArgs[0]);
                    }
                    valuesByQueryAttr.put(entry.getKey(), queryAttrValue);
                }

                final String index = queryArgs.length == 2 ? queryArgs[1] : null;
                if (index != null) {
                    final Long value = valuesByQueryAttr.get(index);
                    if (value == null) {
                        this.shell.getPrintStream().println(String.format("%d", 0));
                    } else {
                        this.shell.getPrintStream().println(String.format("%s", value));
                    }
                } else {
                    for (final Map.Entry<String, Long> entry : valuesByQueryAttr.entrySet()) {
                        this.shell.getPrintStream().println(String.format("%s!%d", entry.getKey(), entry.getValue()));
                    }
                }
            }
        }
    }

    /**
     * Print the full table
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     */
    private void fullTable() throws MonitoringSubFunctionException {

        try {
            final JMXClient client = getJMXClient();
            final Map<String, OutgoingConnectionObjectPoolMetrics> values = client
                    .getTcpTransporterMonitoringServiceClient()
                    .getOutgoingConnectionPoolMetrics();

            int maxSizeRemoteContainer = 0;
            int maxSizeActiveCurrent = 0;
            int maxSizeActiveMax = 0;
            int maxSizeIdleCurrent = 0;
            int maxSizeIdleMax = 0;
            int maxSizeMin = 0;
            int maxSizeMax = 0;
            int maxSizeEstablished = 0;
            for (final Entry<String, OutgoingConnectionObjectPoolMetrics> entry : values.entrySet()) {
                final OutgoingConnectionObjectPoolMetrics metrics = entry.getValue();
                maxSizeRemoteContainer = Math.max(maxSizeRemoteContainer,
                        entry.getKey() == null ? 0 : entry.getKey().length());
                maxSizeActiveCurrent = Math.max(maxSizeActiveCurrent,
                        metrics.getBorrowedObjectNumber().toString().length());
                maxSizeActiveMax = Math.max(maxSizeActiveMax, metrics.getBorrowedObjectMaxNumber().toString().length());
                maxSizeIdleCurrent = Math.max(maxSizeIdleCurrent, metrics.getIdleObjectNumber().toString().length());
                maxSizeIdleMax = Math.max(maxSizeIdleMax, metrics.getIdleObjectMaxNumber().toString().length());
                maxSizeMin = Math.max(maxSizeMin, metrics.getMinIdleSize().toString().length());
                maxSizeMax = Math.max(maxSizeMax, metrics.getMaxSize().toString().length());
                maxSizeEstablished = Math.max(maxSizeEstablished, metrics.getEstablished().toString().length());
            }

            maxSizeRemoteContainer = Math.max(maxSizeRemoteContainer, HEADER_REMOTE_CONTAINER.length());
            maxSizeActiveCurrent = Math.max(maxSizeActiveCurrent, HEADER_ACTIVE_CONNECTION_CURRENT.length());
            maxSizeActiveMax = Math.max(maxSizeActiveMax, HEADER_ACTIVE_CONNECTION_MAX.length());
            maxSizeIdleCurrent = Math.max(maxSizeIdleCurrent, HEADER_IDLE_CONNECTION_CURRENT.length());
            maxSizeIdleMax = Math.max(maxSizeIdleMax, HEADER_IDLE_CONNECTION_MAX.length());
            maxSizeMin = Math.max(maxSizeMin, HEADER_MIN_SIZE.length());
            maxSizeMax = Math.max(maxSizeMax, HEADER_MAX_SIZE.length());
            maxSizeEstablished = Math.max(maxSizeEstablished, HEADER_ESTABLISHED_CONNECTIONS.length());

            final String rowHeaderFormat = "| %-" + maxSizeRemoteContainer + "s || %-" + maxSizeActiveCurrent + "s | %-"
                    + maxSizeActiveMax + "s | %-" + maxSizeIdleCurrent + "s | %-" + maxSizeIdleMax + "s | %-"
                    + maxSizeMin + "s | %-" + maxSizeMax + "s | %-" + maxSizeEstablished + "s |";
            final String header = String.format(rowHeaderFormat, HEADER_REMOTE_CONTAINER,
                    HEADER_ACTIVE_CONNECTION_CURRENT, HEADER_ACTIVE_CONNECTION_MAX, HEADER_IDLE_CONNECTION_CURRENT,
                    HEADER_IDLE_CONNECTION_MAX, HEADER_MIN_SIZE, HEADER_MAX_SIZE, HEADER_ESTABLISHED_CONNECTIONS);
            this.shell.getPrintStream().println(header);
            for (int i = 0; i < header.length(); i++) {
                this.shell.getPrintStream().append('-');
            }
            this.shell.getPrintStream().append('\n');

            final String rowFormat = "| %-" + maxSizeRemoteContainer + "s || %-" + maxSizeActiveCurrent + "d | %-"
                    + maxSizeActiveMax + "d | %-" + maxSizeIdleCurrent + "d | %-" + maxSizeIdleMax + "d | %-"
                    + maxSizeMin + "d | %" + maxSizeMax + "d | %" + maxSizeEstablished + "d |";
            for (final Entry<String, OutgoingConnectionObjectPoolMetrics> entry : values.entrySet()) {

                final OutgoingConnectionObjectPoolMetrics metrics = entry.getValue();

                this.shell.getPrintStream()
                        .println(String.format(rowFormat, entry.getKey(), metrics.getBorrowedObjectNumber(),
                                metrics.getBorrowedObjectMaxNumber(), metrics.getIdleObjectNumber(),
                                metrics.getIdleObjectMaxNumber(), metrics.getMinIdleSize(), metrics.getMaxSize(),
                                metrics.getEstablished()));
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final TcpTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final TcpTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

}
