/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.INCOMING_WS_REQUEST_COUNT;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.IncomingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.WsRequestExecutionStatus;
import org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock;

/**
 * Unit tests of {@link IncomingWsRequestsCount}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class IncomingWsRequestsCountTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new IncomingWsRequestsCount();
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_02() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-"
                    + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option", IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    e.getOption().getOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_03() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_04() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsCount.LIST_OPERATIONS_LONG_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_05() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsCount.LIST_OPERATIONS_LONG_OPTION,
                    "--" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                    IncomingWsRequestsCount.QUERY_ATTR_NAME });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_06() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsCount.LIST_OPERATIONS_LONG_OPTION,
                    "-" + IncomingWsRequestsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_07() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option having a missing argument is not the expected one",
                    IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION, e.getOption()
                            .getLongOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with an invalid argument.
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void testExecute_Error_08() throws MonitoringSubFunctionException {

        final String invalidArg = "invalid-arg";
        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION, invalidArg });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("The option having an invalid argument is not the expected one", IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION, e.getOption().getLongOpt());
            assertEquals("The invalid value is not the expected one", invalidArg, e.getValue());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_09() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                    IncomingWsRequestsCount.QUERY_ATTR_NAME,
                    "-" + IncomingWsRequestsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-t' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_10() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + IncomingWsRequestsCount.TABLE_SHORT_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--list-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + IncomingWsRequestsCount.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_01() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "--" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + IncomingWsRequestsCount.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_03() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + IncomingWsRequestsCount.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_04() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "--" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_05() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + IncomingWsRequestsCount.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the incoming ws-request counter,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException {

        INCOMING_WS_REQUEST_COUNT = null;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>a null value is returned for the metric associated to the incoming
     * ws-request counter,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException {

        INCOMING_WS_REQUEST_COUNT = null;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the incoming
     * ws-request counter,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException,
            IOException {

        INCOMING_WS_REQUEST_COUNT = null;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.TABLE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        final String header = brOutput.readLine();
        assertTrue("No header", header != null && !header.isEmpty());
        final String sepLine = brOutput.readLine();
        assertTrue("No line", sepLine != null && !sepLine.isEmpty());
        final String end = brOutput.readLine();
        assertNull("Extra character displayed: [" + end + "]", end);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-list-operations' is provided without argument,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException,
            IOException {

        final long incomingWsRequestCount1 = 789;
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final long incomingWsRequestCount2 = 10;
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, "clientIpAddress",
                WsRequestExecutionStatus.SUCCEEDED),
                incomingWsRequestCount1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, "clientIpAddress",
                WsRequestExecutionStatus.SUCCEEDED),
                incomingWsRequestCount2);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            boolean operationFound = false;
            for (final Map.Entry<IncomingWsRequestKey, Long> entry : INCOMING_WS_REQUEST_COUNT
                    .entrySet()) {
                final IncomingWsRequestKey key = entry.getKey();
                if (line.equals(key.getWsOperation() + "@" + key.getWsPath())) {
                    operationFound = true;
                    nbOperations++;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
        }
        assertEquals("unexpected operation number", INCOMING_WS_REQUEST_COUNT.size(), nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-query-operations' is provided with the argument "NAME",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException,
            IOException {

        final long incomingWsRequestCountSucceeded1 = 789;
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final String incomingClientIpAddress1 = "clientIpAddress1";
        final long incomingWsRequestCountSucceeded2 = 10;
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final String incomingClientIpAddress2 = "clientIpAddress2";
        final long incomingWsRequestCountSucceeded3 = 56;
        final long incomingWsRequestCountFault1 = 56;
        final long incomingWsRequestCountFault2 = 12;
        final long incomingWsRequestCountFault3 = 34;
        final long incomingWsRequestCountError1 = 9;
        final long incomingWsRequestCountError2 = 7;

        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError1);

        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                Long.valueOf(incomingWsRequestCountSucceeded1));
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                Long.valueOf(incomingWsRequestCountSucceeded2 + incomingWsRequestCountSucceeded3));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<IncomingWsRequestKey, Long> entry : INCOMING_WS_REQUEST_COUNT
                    .entrySet()) {
                final IncomingWsRequestKey key = entry.getKey();
                if (operationNameReturned.equals(key.getWsOperation() + "@" + key.getWsPath())) {
                    operationFound = true;
                    assertEquals("Unexpected name value", operationNameReturned, value);
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     * @throws ConnectionErrorException 
     */
    @Test
    public void testExecute_11() throws MonitoringSubFunctionException,
            IOException, ConnectionErrorException {

        final long incomingWsRequestCountSucceeded1 = 789;
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final String incomingClientIpAddress1 = "clientIpAddress1";
        final long incomingWsRequestCountSucceeded2 = 10;
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final String incomingClientIpAddress2 = "clientIpAddress2";
        final long incomingWsRequestCountSucceeded3 = 56;
        final long incomingWsRequestCountFault1 = 56;
        final long incomingWsRequestCountFault2 = 12;
        final long incomingWsRequestCountFault3 = 34;
        final long incomingWsRequestCountError1 = 9;
        final long incomingWsRequestCountError2 = 7;

        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError1);

        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                Long.valueOf(incomingWsRequestCountSucceeded1));
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                Long.valueOf(incomingWsRequestCountSucceeded2 + incomingWsRequestCountSucceeded3));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_SUCCEEDED });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingCountOutput(output, INCOMING_WS_REQUEST_COUNT, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the arguments
     * "SUCCEEDED" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_12() throws MonitoringSubFunctionException,
            IOException {

        final long incomingWsRequestCountSucceeded1 = 789;
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final String incomingClientIpAddress1 = "clientIpAddress1";
        final long incomingWsRequestCountSucceeded2 = 10;
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final String incomingClientIpAddress2 = "clientIpAddress2";
        final long incomingWsRequestCountSucceeded3 = 56;
        final long incomingWsRequestCountFault1 = 56;
        final long incomingWsRequestCountFault2 = 12;
        final long incomingWsRequestCountFault3 = 34;
        final long incomingWsRequestCountError1 = 9;
        final long incomingWsRequestCountError2 = 7;

        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError1);

        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                Long.valueOf(incomingWsRequestCountSucceeded2 + incomingWsRequestCountSucceeded3));

        final String queriedOperation = incomingWsRequestWsOperation2 + "@"
                + incomingWsRequestWsPath2;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_SUCCEEDED, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected succeeded value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     * @throws ConnectionErrorException 
     */
    @Test
    public void testExecute_13() throws MonitoringSubFunctionException,
            IOException, ConnectionErrorException {

        final long incomingWsRequestCountSucceeded1 = 789;
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final String incomingClientIpAddress1 = "clientIpAddress1";
        final long incomingWsRequestCountSucceeded2 = 10;
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final String incomingClientIpAddress2 = "clientIpAddress2";
        final long incomingWsRequestCountSucceeded3 = 56;
        final long incomingWsRequestCountFault1 = 56;
        final long incomingWsRequestCountFault2 = 12;
        final long incomingWsRequestCountFault3 = 34;
        final long incomingWsRequestCountError1 = 9;
        final long incomingWsRequestCountError2 = 7;

        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError1);

        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                Long.valueOf(incomingWsRequestCountFault1 + incomingWsRequestCountFault2));
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                Long.valueOf(incomingWsRequestCountFault3));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_FAULT });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingCountOutput(output, INCOMING_WS_REQUEST_COUNT, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the arguments
     * "FAULT" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_14() throws MonitoringSubFunctionException,
            IOException {

        final long incomingWsRequestCountSucceeded1 = 789;
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final String incomingClientIpAddress1 = "clientIpAddress1";
        final long incomingWsRequestCountSucceeded2 = 10;
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final String incomingClientIpAddress2 = "clientIpAddress2";
        final long incomingWsRequestCountSucceeded3 = 56;
        final long incomingWsRequestCountFault1 = 56;
        final long incomingWsRequestCountFault2 = 12;
        final long incomingWsRequestCountFault3 = 34;
        final long incomingWsRequestCountError1 = 9;
        final long incomingWsRequestCountError2 = 7;

        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError1);

        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                Long.valueOf(incomingWsRequestCountFault3));

        final String queriedOperation = incomingWsRequestWsOperation2 + "@"
                + incomingWsRequestWsPath2;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_FAULT, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected succeeded value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     * @throws ConnectionErrorException 
     */
    @Test
    public void testExecute_15() throws MonitoringSubFunctionException,
            IOException, ConnectionErrorException {

        final long incomingWsRequestCountSucceeded1 = 789;
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final String incomingClientIpAddress1 = "clientIpAddress1";
        final long incomingWsRequestCountSucceeded2 = 10;
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final String incomingClientIpAddress2 = "clientIpAddress2";
        final long incomingWsRequestCountSucceeded3 = 56;
        final long incomingWsRequestCountFault1 = 56;
        final long incomingWsRequestCountFault2 = 12;
        final long incomingWsRequestCountFault3 = 34;
        final long incomingWsRequestCountError1 = 9;
        final long incomingWsRequestCountError2 = 7;

        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError1);

        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                Long.valueOf(incomingWsRequestCountError1));
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                Long.valueOf(incomingWsRequestCountError2));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_ERROR });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingCountOutput(output, INCOMING_WS_REQUEST_COUNT, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the arguments
     * "ERROR" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_16() throws MonitoringSubFunctionException,
            IOException {

        final long incomingWsRequestCountSucceeded1 = 789;
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final String incomingClientIpAddress1 = "clientIpAddress1";
        final long incomingWsRequestCountSucceeded2 = 10;
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final String incomingClientIpAddress2 = "clientIpAddress2";
        final long incomingWsRequestCountSucceeded3 = 56;
        final long incomingWsRequestCountFault1 = 56;
        final long incomingWsRequestCountFault2 = 12;
        final long incomingWsRequestCountFault3 = 34;
        final long incomingWsRequestCountError1 = 9;
        final long incomingWsRequestCountError2 = 7;

        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, incomingClientIpAddress1,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError1);

        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress1,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded2);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestCountSucceeded3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.FAULT), incomingWsRequestCountFault3);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey(incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, incomingClientIpAddress2,
                WsRequestExecutionStatus.ERROR), incomingWsRequestCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                Long.valueOf(incomingWsRequestCountError2));

        final String queriedOperation = incomingWsRequestWsOperation2 + "@"
                + incomingWsRequestWsPath2;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsCount.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsCount.QUERY_ATTR_ERROR, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected succeeded value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }
}
