/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_MAX;

import java.util.List;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;

/**
 * Unit tests of {@link MexAcceptorPool}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MexAcceptorPoolTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new MexAcceptorPool();
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        subFunction.execute(new String[] { "-" + MexAcceptorPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>error occurs getting HTTP thread pool value.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_01() throws MonitoringSubFunctionException {

        subFunction.execute(new String[] { "-" + MexAcceptorPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max active threads of message exchange processor
     * thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max active threads of message exchange acceptor is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException {

        MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Integer.valueOf(123);
        MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_MAX = null;
        subFunction.execute(new String[] { "-" + MexAcceptorPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse(
                "the metric value associated to the max active threads of message exchange acceptor pool is displayed",
                output.contains(MexAcceptorPool.HEADER_ACTIVE_MAX));
        assertTrue(
                "the metric value associated to the current active threads of message exchange acceptor pool is not displayed",
                output.contains(MexAcceptorPool.HEADER_ACTIVE_CURRENT));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current active threads of message exchange acceptor
     * thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current active threads of the message exchange acceptor thread pool is not
     * returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_03() throws MonitoringSubFunctionException {

        MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = null;
        MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Integer.valueOf(123);
        subFunction.execute(new String[] { "-" + MexAcceptorPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of message exchange acceptor thread pool is not displayed",
                output.contains(MexAcceptorPool.HEADER_ACTIVE_MAX));
        assertFalse(
                "the metric value associated to the current active threads of message exchange acceptor thread pool is displayed",
                output.contains(MexAcceptorPool.HEADER_ACTIVE_CURRENT));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>no null value is returned for metric,</li>
     * <li>no error will occur.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException {

        final int mexAcceptorThreadPoolActiveThreadCurrent = 123;
        final int mexAcceptorThreadPoolActiveThreadMax = 167;
        MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Integer.valueOf(mexAcceptorThreadPoolActiveThreadCurrent);
        MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Integer.valueOf(mexAcceptorThreadPoolActiveThreadMax);
        subFunction.execute(new String[] { "-" + MexAcceptorPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        boolean isMissingMexAcceptorPoolActiveThreadCurrent = true;
        boolean isMissingMexAcceptorPoolActiveThreadMax = true;
        final StringTokenizer strTokenizer = new StringTokenizer(output);
        while (strTokenizer.hasMoreElements()) {
            final StringTokenizer metricTokenizer = new StringTokenizer(strTokenizer.nextToken(), ":");
            final String metricLabel = metricTokenizer.nextToken();
            assertNotNull("missing metric label", metricLabel);
            final String metricValue = metricTokenizer.nextToken();
            assertNotNull("missing metric value", metricValue);
            if (MexAcceptorPool.HEADER_ACTIVE_CURRENT.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current active threads of message exchange acceptor thread pool",
                        MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT, Integer.valueOf(metricValue));
                isMissingMexAcceptorPoolActiveThreadCurrent = false;
            } else if (MexAcceptorPool.HEADER_ACTIVE_MAX.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max active threads of message exchange acceptor thread pool",
                        MEX_ACCEPTOR_THREAD_POOL_ACTIVE_THREAD_MAX, Integer.valueOf(metricValue));
                isMissingMexAcceptorPoolActiveThreadMax = false;
            } else {
                fail("Unexpected metric");
            }
        }
        assertFalse(
                "The metric value associated to the current active threads of message exchange acceptor thread pool is missing",
                isMissingMexAcceptorPoolActiveThreadCurrent);
        assertFalse(
                "The metric value associated to the max active threads of message exchange acceptor thread pool is missing",
                isMissingMexAcceptorPoolActiveThreadMax);
    }
}
