/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_THREAD_POOL_MAX_SIZE;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.MEX_PROCESSOR_THREAD_POOL_MIN_SIZE;

import java.util.List;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;

/**
 * Unit tests of {@link MexProcessorThreadPool}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MexProcessorThreadPoolTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new MexProcessorThreadPool();
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>error occurs getting message exchange processor thread pool values.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_01() throws MonitoringSubFunctionException {

        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max active threads of message exchange processor
     * thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max active threads of message exchange processor thread pool is not
     * returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException {

        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = null;
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse(
                "the metric value associated to the max active threads of message exchange processor thread pool is displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current active threads of message exchange
     * processor thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current active threads of message exchange processor thread pool is not
     * returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_03() throws MonitoringSubFunctionException {

        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = null;
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertFalse(
                "the metric value associated to the current active threads of message exchange processor thread pool is displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current
     * idle threads of message exchange processor thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current idle threads of message
     * exchange processor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_04() throws MonitoringSubFunctionException {

        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = null;
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertFalse(
                "the metric value associated to the current idle threads of message exchange processor thread pool is displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max idle
     * threads of message exchange processor thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max idle threads of message
     * exchange processor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_05() throws MonitoringSubFunctionException {

        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = null;
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertFalse(
                "the metric value associated to the max idle threads of message exchange processor thread pool is displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max
     * enqueued request number in message exchange processor thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max enqueued request number in
     * message exchange processor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException {

        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = null;
        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertFalse(
                "the metric value associated to the max enqueued request number in the message exchange processor thread pool is displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current
     * enqueued request number in message exchange processor thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current enqueued request number in
     * message exchange processor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException {

        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = null;
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of message exchange processor thread pool is ot displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MIN_SIZE));
        assertFalse(
                "the metric value associated to the current enqueued request number in the message exchange processor thread pool is displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max size of
     * message exchange processor thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max size of message exchange
     * processor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException {

        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = null;
        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertFalse(
                "the metric value associated to the max size of message exchange processor thread pool is displayed",
                output.contains(MexProcessorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the min size of
     * message exchange processor thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the min size of message exchange
     * processor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException {

        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = null;
        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_MAX_SIZE));
        assertFalse(
                "the metric value associated to the min size of message exchange processor thread pool is displayed",
                output.contains(MexProcessorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the message exchange processor thread pool is not displayed",
                output.contains(MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>no null value is returned for metric,</li>
     * <li>no error will occur.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException {

        final long httpServerThreadPoolActiveThreadCurrent = 123;
        final long httpServerThreadPoolActiveThreadMax = 167;
        final long httpServerThreadPoolIdleThreadCurrent = 2;
        final long httpServerThreadPoolIdleThreadMax = 7;
        final long httpServerThreadPoolEnqueuedRequestCurrent = 1;
        final long httpServerThreadPoolEnqueuedRequestMax = 3;
        final long httpServerThreadPoolMinSize = 5;
        final long httpServerThreadPoolMaxSize = 25;
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(httpServerThreadPoolActiveThreadCurrent);
        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(httpServerThreadPoolActiveThreadMax);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long
                .valueOf(httpServerThreadPoolIdleThreadCurrent);
        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(httpServerThreadPoolIdleThreadMax);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long
                .valueOf(httpServerThreadPoolEnqueuedRequestCurrent);
        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long
                .valueOf(httpServerThreadPoolEnqueuedRequestMax);
        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = Long.valueOf(httpServerThreadPoolMinSize);
        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = Long.valueOf(httpServerThreadPoolMaxSize);
        subFunction.execute(new String[] {
                "-" + MexProcessorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        boolean isMissingMexProcessorThreadPoolActiveThreadCurrent = true;
        boolean isMissingMexProcessorThreadPoolActiveThreadMax = true;
        boolean isMissingMexProcessorThreadPoolIdleThreadCurrent = true;
        boolean isMissingMexProcessorThreadPoolIdleThreadMax = true;
        boolean isMissingMexProcessorThreadPoolEnqueuedRequestMax = true;
        boolean isMissingMexProcessorThreadPoolEnqueuedRequestCurrent = true;
        boolean isMissingMexProcessorThreadPoolMinSize = true;
        boolean isMissingMexProcessorThreadPoolMaxSize = true;
        final StringTokenizer strTokenizer = new StringTokenizer(output);
        while (strTokenizer.hasMoreElements()) {
            final StringTokenizer metricTokenizer = new StringTokenizer(strTokenizer.nextToken(),
                    ":");
            final String metricLabel = metricTokenizer.nextToken();
            assertNotNull("missing metric label", metricLabel);
            final String metricValue = metricTokenizer.nextToken();
            assertNotNull("missing metric value", metricValue);
            if (MexProcessorThreadPool.HEADER_ACTIVE_THREAD_CURRENT
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current active threads of message exchange processor thread pool",
                        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_CURRENT,
                        Long.valueOf(metricValue));
                isMissingMexProcessorThreadPoolActiveThreadCurrent = false;
            } else if (MexProcessorThreadPool.HEADER_ACTIVE_THREAD_MAX
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max active threads of message exchange processor thread pool",
                        MEX_PROCESSOR_THREAD_POOL_ACTIVE_THREAD_MAX, Long.valueOf(metricValue));
                isMissingMexProcessorThreadPoolActiveThreadMax = false;
            } else if (MexProcessorThreadPool.HEADER_IDLE_THREAD_MAX
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max idle threads of message exchange processor thread pool",
                        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_MAX, Long.valueOf(metricValue));
                isMissingMexProcessorThreadPoolIdleThreadMax = false;
            } else if (MexProcessorThreadPool.HEADER_IDLE_THREAD_CURRENT
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current idle threads of message exchange processor thread pool",
                        MEX_PROCESSOR_THREAD_POOL_IDLE_THREAD_CURRENT, Long.valueOf(metricValue));
                isMissingMexProcessorThreadPoolIdleThreadCurrent = false;
            } else if (MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_MAX
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max enqueued request in the message exchange processor thread pool",
                        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_MAX, Long.valueOf(metricValue));
                isMissingMexProcessorThreadPoolEnqueuedRequestMax = false;
            } else if (MexProcessorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT
                    .equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current enqueued request into the message exchange processor thread pool",
                        MEX_PROCESSOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT,
                        Long.valueOf(metricValue));
                isMissingMexProcessorThreadPoolEnqueuedRequestCurrent = false;
            } else if (MexProcessorThreadPool.HEADER_MIN_SIZE.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the min size of the message exchange processor thread pool",
                        MEX_PROCESSOR_THREAD_POOL_MIN_SIZE, Long.valueOf(metricValue));
                isMissingMexProcessorThreadPoolMinSize = false;
            } else if (MexProcessorThreadPool.HEADER_MAX_SIZE.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max size of the message exchange processor thread pool",
                        MEX_PROCESSOR_THREAD_POOL_MAX_SIZE, Long.valueOf(metricValue));
                isMissingMexProcessorThreadPoolMaxSize = false;
            } else {
                fail("Unexpected metric");
            }
        }
        assertFalse(
                "The metric value associated to the current active threads of message exchange processor thread pool is missing",
                isMissingMexProcessorThreadPoolActiveThreadCurrent);
        assertFalse(
                "The metric value associated to the max active threads of message exchange processor thread pool is missing",
                isMissingMexProcessorThreadPoolActiveThreadMax);
        assertFalse(
                "The metric value associated to the current idle threads of message exchange processor thread pool is missing",
                isMissingMexProcessorThreadPoolIdleThreadCurrent);
        assertFalse(
                "The metric value associated to the max idle threads of message exchange processor thread pool is missing",
                isMissingMexProcessorThreadPoolIdleThreadMax);
        assertFalse(
                "The metric value associated to the current enqueued request into the message exchange processor thread pool is missing",
                isMissingMexProcessorThreadPoolEnqueuedRequestCurrent);
        assertFalse(
                "The metric value associated to the max enqueued request into the message exchange processor thread pool is missing",
                isMissingMexProcessorThreadPoolEnqueuedRequestMax);
        assertFalse(
                "The metric value associated the max size of message exchange processor thread pool is missing",
                isMissingMexProcessorThreadPoolMaxSize);
        assertFalse(
                "The metric value associated the min size of message exchange processor thread pool is missing",
                isMissingMexProcessorThreadPoolMinSize);
    }
}
