/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.SERVICE_PROVIDER_INVOCATIONS_COUNT;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationKey;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationStatus;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;

/**
 * Unit tests of {@link ServiceProviderInvocationsCount}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ServiceProviderInvocationsCountTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new ServiceProviderInvocationsCount();
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_02() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] { "-"
                    + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option",
                    ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                    e.getOption().getOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_03() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {
                    "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_04() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {
                            "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvocationsCount.LIST_OPERATIONS_LONG_OPTION,
                            "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_05() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {
                    "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvocationsCount.LIST_OPERATIONS_LONG_OPTION,
                    "--" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                    ServiceProviderInvocationsCount.QUERY_ATTR_NAME });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_06() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {
                    "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvocationsCount.LIST_OPERATIONS_LONG_OPTION,
                    "-" + ServiceProviderInvocationsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_07() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {
                    "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option having a missing argument is not the expected one",
                    ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION, e.getOption()
                            .getLongOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with an invalid argument.
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void testExecute_Error_08() throws MonitoringSubFunctionException {

        final String invalidArg = "invalid-arg";
        try {
            subFunction.execute(new String[] {
                            "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                            invalidArg });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("The option having an invalid argument is not the expected one",
                    ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION, e.getOption()
                            .getLongOpt());
            assertEquals("The invalid value is not the expected one", invalidArg, e.getValue());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_09() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {
                    "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                    ServiceProviderInvocationsCount.QUERY_ATTR_NAME,
                    "-" + ServiceProviderInvocationsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-t' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_10() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {
                    "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                    ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + ServiceProviderInvocationsCount.TABLE_SHORT_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist,</li>
     * <li>the option '--list-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + ServiceProviderInvocationsCount.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_01() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "--" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided does not exist,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + ServiceProviderInvocationsCount.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_03() throws MonitoringSubFunctionException {

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + ServiceProviderInvocationsCount.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_04() throws MonitoringSubFunctionException {

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "--" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_05() throws MonitoringSubFunctionException {

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + ServiceProviderInvocationsCount.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the incoming
     * ws-request counter,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException {

        SERVICE_PROVIDER_INVOCATIONS_COUNT = null;
        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>a null value is returned for the metric associated to the incoming
     * ws-request counter,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException {

        SERVICE_PROVIDER_INVOCATIONS_COUNT = null;
        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the incoming
     * ws-request counter,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException, IOException {

        SERVICE_PROVIDER_INVOCATIONS_COUNT = null;
        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.TABLE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        final String header = brOutput.readLine();
        assertTrue("No header", header != null && !header.isEmpty());
        final String sepLine = brOutput.readLine();
        assertTrue("No line", sepLine != null && !sepLine.isEmpty());
        final String end = brOutput.readLine();
        assertNull("Extra character displayed: [" + end + "]", end);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '-list-operations' is provided without argument,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException, IOException {

        final long serviceProviderInvocationsCount1 = 789;
        final String serviceProviderInvocationsItfName1 = "itfName-1";
        final String serviceProviderInvocationsSvcName1 = "svcName-1";
        final String serviceProviderInvocationsOperation1 = "operation1";
        final long serviceProviderInvocationsCount2 = 10;
        final String serviceProviderInvocationsItfName2 = "itfName-2";
        final String serviceProviderInvocationsSvcName2 = "svcName-2";
        final String serviceProviderInvocationsOperation2 = "operation2";
        SERVICE_PROVIDER_INVOCATIONS_COUNT = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1, "mep",
                ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationsCount1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2, "mep",
                ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationsCount2);

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvocationKey, Long> entry : SERVICE_PROVIDER_INVOCATIONS_COUNT
                    .entrySet()) {
                final ServiceProviderInvocationKey key = entry.getKey();
                if (line.equals(key.getOperation() + "@" + key.getServiceName() + "@"
                        + key.getInterfaceName())) {
                    operationFound = true;
                    nbOperations++;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
        }
        assertEquals("unexpected operation number", SERVICE_PROVIDER_INVOCATIONS_COUNT.size(),
                nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '-query-operations' is provided with the argument "NAME",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException, IOException {

        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvocationsItfName1 = "itfName-1";
        final String serviceProviderInvocationsSvcName1 = "svcName-1";
        final String serviceProviderInvocationsOperation1 = "operation1";
        final String serviceProviderInvocationsMep1 = "mep1";
        final long serviceProviderInvocationsCountSucceeded2 = 10;
        final String serviceProviderInvocationsItfName2 = "itfName-2";
        final String serviceProviderInvocationsSvcName2 = "svcName-2";
        final String serviceProviderInvocationsOperation2 = "operation2";
        final String serviceProviderInvocationsMep2 = "mep2";
        final long serviceProviderInvocationsCountSucceeded3 = 56;
        final long serviceProviderInvocationsCountFault1 = 56;
        final long serviceProviderInvocationsCountFault2 = 12;
        final long serviceProviderInvocationsCountFault3 = 34;
        final long serviceProviderInvocationsCountError1 = 9;
        final long serviceProviderInvocationsCountError2 = 7;

        SERVICE_PROVIDER_INVOCATIONS_COUNT = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError1);

        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationsOperation1 + "@"
                + serviceProviderInvocationsSvcName1,
                Long.valueOf(outgoingWsRequestCountSucceeded1));
        expectedResults.put(
                serviceProviderInvocationsOperation2 + "@" + serviceProviderInvocationsSvcName2,
                Long.valueOf(serviceProviderInvocationsCountSucceeded2
                        + serviceProviderInvocationsCountSucceeded3));

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvocationKey, Long> entry : SERVICE_PROVIDER_INVOCATIONS_COUNT
                    .entrySet()) {
                final ServiceProviderInvocationKey key = entry.getKey();
                if (operationNameReturned.equals(key.getOperation() + "@" + key.getServiceName()
                        + "@" + key.getInterfaceName())) {
                    operationFound = true;
                    assertEquals("Unexpected name value", operationNameReturned, value);
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_11() throws MonitoringSubFunctionException, ConnectionErrorException,
            IOException {

        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvocationsItfName1 = "itfName-1";
        final String serviceProviderInvocationsSvcName1 = "svcName-1";
        final String serviceProviderInvocationsOperation1 = "operation1";
        final String serviceProviderInvocationsMep1 = "mep1";
        final long serviceProviderInvocationsCountSucceeded2 = 10;
        final String serviceProviderInvocationsItfName2 = "itfName-2";
        final String serviceProviderInvocationsSvcName2 = "svcName-2";
        final String serviceProviderInvocationsOperation2 = "operation2";
        final String serviceProviderInvocationsMep2 = "mep2";
        final long serviceProviderInvocationsCountSucceeded3 = 56;
        final long serviceProviderInvocationsCountFault1 = 56;
        final long serviceProviderInvocationsCountFault2 = 12;
        final long serviceProviderInvocationsCountFault3 = 34;
        final long serviceProviderInvocationsCountError1 = 9;
        final long serviceProviderInvocationsCountError2 = 7;

        SERVICE_PROVIDER_INVOCATIONS_COUNT = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError1);

        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationsOperation1 + "@"
                + serviceProviderInvocationsSvcName1 + "@" + serviceProviderInvocationsItfName1,
                Long.valueOf(outgoingWsRequestCountSucceeded1));
        expectedResults.put(
                serviceProviderInvocationsOperation2 + "@" + serviceProviderInvocationsSvcName2
                        + "@" + serviceProviderInvocationsItfName2,
                Long.valueOf(serviceProviderInvocationsCountSucceeded2
                        + serviceProviderInvocationsCountSucceeded3));

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_SUCCEEDED });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationsCountOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_COUNT,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the arguments
     * "SUCCEEDED" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_12() throws MonitoringSubFunctionException, IOException {

        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvocationsItfName1 = "itfName-1";
        final String serviceProviderInvocationsSvcName1 = "svcName-1";
        final String serviceProviderInvocationsOperation1 = "operation1";
        final String serviceProviderInvocationsMep1 = "mep1";
        final long serviceProviderInvocationsCountSucceeded2 = 10;
        final String serviceProviderInvocationsItfName2 = "itfName-2";
        final String serviceProviderInvocationsSvcName2 = "svcName-2";
        final String serviceProviderInvocationsOperation2 = "operation2";
        final String serviceProviderInvocationsMep2 = "mep2";
        final long serviceProviderInvocationsCountSucceeded3 = 56;
        final long serviceProviderInvocationsCountFault1 = 56;
        final long serviceProviderInvocationsCountFault2 = 12;
        final long serviceProviderInvocationsCountFault3 = 34;
        final long serviceProviderInvocationsCountError1 = 9;
        final long serviceProviderInvocationsCountError2 = 7;

        SERVICE_PROVIDER_INVOCATIONS_COUNT = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError1);

        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationsOperation2 + "@" + serviceProviderInvocationsSvcName2
                        + "@" + serviceProviderInvocationsItfName2,
                Long.valueOf(serviceProviderInvocationsCountSucceeded2
                        + serviceProviderInvocationsCountSucceeded3));

        final String queriedOperation = serviceProviderInvocationsOperation2 + "@"
                + serviceProviderInvocationsSvcName2 + "@" + serviceProviderInvocationsItfName2;
        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_SUCCEEDED, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected succeeded value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_13() throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvocationsItfName1 = "itfName-1";
        final String serviceProviderInvocationsSvcName1 = "svcName-1";
        final String serviceProviderInvocationsOperation1 = "operation1";
        final String serviceProviderInvocationsMep1 = "mep1";
        final long serviceProviderInvocationsCountSucceeded2 = 10;
        final String serviceProviderInvocationsItfName2 = "itfName-2";
        final String serviceProviderInvocationsSvcName2 = "svcName-2";
        final String serviceProviderInvocationsOperation2 = "operation2";
        final String serviceProviderInvocationsMep2 = "mep2";
        final long serviceProviderInvocationsCountSucceeded3 = 56;
        final long serviceProviderInvocationsCountFault1 = 56;
        final long serviceProviderInvocationsCountFault2 = 12;
        final long serviceProviderInvocationsCountFault3 = 34;
        final long serviceProviderInvocationsCountError1 = 9;
        final long serviceProviderInvocationsCountError2 = 7;

        SERVICE_PROVIDER_INVOCATIONS_COUNT = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError1);

        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationsOperation1 + "@" + serviceProviderInvocationsSvcName1
                        + "@" + serviceProviderInvocationsItfName1,
                Long.valueOf(serviceProviderInvocationsCountFault1
                        + serviceProviderInvocationsCountFault2));
        expectedResults.put(serviceProviderInvocationsOperation2 + "@"
                + serviceProviderInvocationsSvcName2 + "@" + serviceProviderInvocationsItfName2,
                Long.valueOf(serviceProviderInvocationsCountFault3));

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_FAULT });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationsCountOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_COUNT,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the arguments
     * "FAULT" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_14() throws MonitoringSubFunctionException, IOException {

        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvocationsItfName1 = "itfName-1";
        final String serviceProviderInvocationsSvcName1 = "svcName-1";
        final String serviceProviderInvocationsOperation1 = "operation1";
        final String serviceProviderInvocationsMep1 = "mep1";
        final long serviceProviderInvocationsCountSucceeded2 = 10;
        final String serviceProviderInvocationsItfName2 = "itfName-2";
        final String serviceProviderInvocationsSvcName2 = "svcName-2";
        final String serviceProviderInvocationsOperation2 = "operation2";
        final String serviceProviderInvocationsMep2 = "mep2";
        final long serviceProviderInvocationsCountSucceeded3 = 56;
        final long serviceProviderInvocationsCountFault1 = 56;
        final long serviceProviderInvocationsCountFault2 = 12;
        final long serviceProviderInvocationsCountFault3 = 34;
        final long serviceProviderInvocationsCountError1 = 9;
        final long serviceProviderInvocationsCountError2 = 7;

        SERVICE_PROVIDER_INVOCATIONS_COUNT = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1,
                serviceProviderInvocationsSvcName1, serviceProviderInvocationsOperation1,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError1);

        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep1, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.FAULT),
                serviceProviderInvocationsCountFault3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2,
                serviceProviderInvocationsSvcName2, serviceProviderInvocationsOperation2,
                serviceProviderInvocationsMep2, ServiceProviderInvocationStatus.ERROR),
                serviceProviderInvocationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationsOperation2 + "@"
                + serviceProviderInvocationsSvcName2 + "@" + serviceProviderInvocationsItfName2,
                Long.valueOf(serviceProviderInvocationsCountFault3));

        final String queriedOperation = serviceProviderInvocationsOperation2 + "@"
                + serviceProviderInvocationsSvcName2 + "@" + serviceProviderInvocationsItfName2;
        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_FAULT, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected succeeded value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_15() throws MonitoringSubFunctionException, IOException, ConnectionErrorException {

        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvocationsItfName1 = "itfName-1";
        final String serviceProviderInvocationsSvcName1 = "svcName-1";
        final String serviceProviderInvocationsOperation1 = "operation1";
        final String serviceProviderInvocationsMep1 = "mep1";
        final long serviceProviderInvocationsCountSucceeded2 = 10;
        final String serviceProviderInvocationsItfName2 = "itfName-2";
        final String serviceProviderInvocationsSvcName2 = "svcName-2";
        final String serviceProviderInvocationsOperation2 = "operation2";
        final String serviceProviderInvocationsMep2 = "mep2";
        final long serviceProviderInvocationsCountSucceeded3 = 56;
        final long serviceProviderInvocationsCountFault1 = 56;
        final long serviceProviderInvocationsCountFault2 = 12;
        final long serviceProviderInvocationsCountFault3 = 34;
        final long serviceProviderInvocationsCountError1 = 9;
        final long serviceProviderInvocationsCountError2 = 7;

        SERVICE_PROVIDER_INVOCATIONS_COUNT = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1, serviceProviderInvocationsSvcName1,
                serviceProviderInvocationsOperation1, serviceProviderInvocationsMep1,
                ServiceProviderInvocationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1, serviceProviderInvocationsSvcName1,
                serviceProviderInvocationsOperation1, serviceProviderInvocationsMep1,
                ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationsCountFault1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1, serviceProviderInvocationsSvcName1,
                serviceProviderInvocationsOperation1, serviceProviderInvocationsMep2,
                ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationsCountFault2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1, serviceProviderInvocationsSvcName1,
                serviceProviderInvocationsOperation1, serviceProviderInvocationsMep1,
                ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationsCountError1);

        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2, serviceProviderInvocationsSvcName2,
                serviceProviderInvocationsOperation2, serviceProviderInvocationsMep1,
                ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2, serviceProviderInvocationsSvcName2,
                serviceProviderInvocationsOperation2, serviceProviderInvocationsMep2,
                ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2, serviceProviderInvocationsSvcName2,
                serviceProviderInvocationsOperation2, serviceProviderInvocationsMep2,
                ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationsCountFault3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2, serviceProviderInvocationsSvcName2,
                serviceProviderInvocationsOperation2, serviceProviderInvocationsMep2,
                ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationsOperation1 + "@"
                + serviceProviderInvocationsSvcName1 + "@" + serviceProviderInvocationsItfName1,
                Long.valueOf(serviceProviderInvocationsCountError1));
        expectedResults.put(serviceProviderInvocationsOperation2 + "@"
                + serviceProviderInvocationsSvcName2 + "@" + serviceProviderInvocationsItfName2,
                Long.valueOf(serviceProviderInvocationsCountError2));

        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_ERROR });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationsCountOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_COUNT,
                expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the component identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the arguments
     * "ERROR" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_16() throws MonitoringSubFunctionException, IOException {

        final long outgoingWsRequestCountSucceeded1 = 789;
        final String serviceProviderInvocationsItfName1 = "itfName-1";
        final String serviceProviderInvocationsSvcName1 = "svcName-1";
        final String serviceProviderInvocationsOperation1 = "operation1";
        final String serviceProviderInvocationsMep1 = "mep1";
        final long serviceProviderInvocationsCountSucceeded2 = 10;
        final String serviceProviderInvocationsItfName2 = "itfName-2";
        final String serviceProviderInvocationsSvcName2 = "svcName-2";
        final String serviceProviderInvocationsOperation2 = "operation2";
        final String serviceProviderInvocationsMep2 = "mep2";
        final long serviceProviderInvocationsCountSucceeded3 = 56;
        final long serviceProviderInvocationsCountFault1 = 56;
        final long serviceProviderInvocationsCountFault2 = 12;
        final long serviceProviderInvocationsCountFault3 = 34;
        final long serviceProviderInvocationsCountError1 = 9;
        final long serviceProviderInvocationsCountError2 = 7;

        SERVICE_PROVIDER_INVOCATIONS_COUNT = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1, serviceProviderInvocationsSvcName1,
                serviceProviderInvocationsOperation1, serviceProviderInvocationsMep1,
                ServiceProviderInvocationStatus.SUCCEEDED),
                outgoingWsRequestCountSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1, serviceProviderInvocationsSvcName1,
                serviceProviderInvocationsOperation1, serviceProviderInvocationsMep1,
                ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationsCountFault1);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1, serviceProviderInvocationsSvcName1,
                serviceProviderInvocationsOperation1, serviceProviderInvocationsMep2,
                ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationsCountFault2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName1, serviceProviderInvocationsSvcName1,
                serviceProviderInvocationsOperation1, serviceProviderInvocationsMep1,
                ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationsCountError1);

        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2, serviceProviderInvocationsSvcName2,
                serviceProviderInvocationsOperation2, serviceProviderInvocationsMep1,
                ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2, serviceProviderInvocationsSvcName2,
                serviceProviderInvocationsOperation2, serviceProviderInvocationsMep2,
                ServiceProviderInvocationStatus.SUCCEEDED),
                serviceProviderInvocationsCountSucceeded3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2, serviceProviderInvocationsSvcName2,
                serviceProviderInvocationsOperation2, serviceProviderInvocationsMep2,
                ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationsCountFault3);
        SERVICE_PROVIDER_INVOCATIONS_COUNT.put(new ServiceProviderInvocationKey(
                serviceProviderInvocationsItfName2, serviceProviderInvocationsSvcName2,
                serviceProviderInvocationsOperation2, serviceProviderInvocationsMep2,
                ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationsCountError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationsOperation2 + "@"
                + serviceProviderInvocationsSvcName2 + "@" + serviceProviderInvocationsItfName2,
                Long.valueOf(serviceProviderInvocationsCountError2));

        final String queriedOperation = serviceProviderInvocationsOperation2 + "@"
                + serviceProviderInvocationsSvcName2 + "@" + serviceProviderInvocationsItfName2;
        subFunction.execute(new String[] {
                "-" + ServiceProviderInvocationsCount.COMPONENT_NAME_SHORT_OPTION,
                ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ServiceProviderInvocationsCount.QUERY_OPERATIONS_LONG_OPTION,
                ServiceProviderInvocationsCount.QUERY_ATTR_ERROR, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected succeeded value", expectedResults.get(queriedOperation),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }
}
