/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.se.flowable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE;

import java.util.List;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock;

/**
 * Unit tests of {@link AsyncExecutorThreadPool}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AsyncExecutorThreadPoolTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new AsyncExecutorThreadPool();
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void noOptionProvided() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedOption() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided does not exist.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the non existent component
     * </p>
     */
    @Test
    public void componentIdDoesNotExist() throws Exception {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>error occurs getting asynchronous job executor thread pool values.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void errorGettingValues() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max active threads of asynchronous job executor
     * thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max active threads of asynchronous job executor thread pool is not
     * returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForMaxActiveThreads() throws MonitoringSubFunctionException {

        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = null;
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse(
                "the metric value associated to the max active threads of asynchronous job executor thread pool is displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current active threads of asynchronous job executor
     * thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current active threads of asynchronous job executor thread pool is not
     * returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForCurrentActiveThreads() throws MonitoringSubFunctionException {

        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = null;
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertFalse(
                "the metric value associated to the current active threads of asynchronous job executor thread pool is displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current idle threads of asynchronous job executor
     * thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current idle threads of asynchronous job executor thread pool is not
     * returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForCurrentIdleThreads() throws MonitoringSubFunctionException {

        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = null;
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertFalse(
                "the metric value associated to the current idle threads of asynchronous job executor thread pool is displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max idle threads of asynchronous job executor
     * thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max idle threads of asynchronous job executor thread pool is not returned,
     * </li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForMaxIdleThreads() throws MonitoringSubFunctionException {

        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = null;
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertFalse(
                "the metric value associated to the max idle threads of asynchronous job executor thread pool is displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max enqueued request number in HTTP server thread
     * pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max enqueued request number in asynchronous job executor thread pool is
     * not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForMaxEnqueuedRequests() throws MonitoringSubFunctionException {

        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = null;
        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertFalse(
                "the metric value associated to the max enqueued request number in the asynchronous job executor thread pool is displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current enqueued request number in HTTP server
     * thread pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current enqueued request number in asynchronous job executor thread pool
     * is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForCurrentEnqueuedRequests() throws MonitoringSubFunctionException {

        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = null;
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of asynchronous job executor thread pool is ot displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MIN_SIZE));
        assertFalse(
                "the metric value associated to the current enqueued request number in the asynchronous job executor thread pool is displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max size of asynchronous job executor thread pool,
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max size of asynchronous job executor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForMaxSize() throws MonitoringSubFunctionException {

        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = null;
        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertFalse("the metric value associated to the max size of asynchronous job executor thread pool is displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MAX_SIZE));
        assertTrue(
                "the metric value associated to the min size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the min size of asynchronous job executor thread pool,
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the min size of asynchronous job executor thread pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForMinSize() throws MonitoringSubFunctionException {

        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(123);
        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = null;
        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current active threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_MAX));
        assertTrue(
                "the metric value associated to the current idle threads of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_IDLE_THREAD_CURRENT));
        assertTrue(
                "the metric value associated to the max size of asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MAX_SIZE));
        assertFalse("the metric value associated to the min size of asynchronous job executor thread pool is displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_MIN_SIZE));
        assertTrue(
                "the metric value associated to the current enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT));
        assertTrue(
                "the metric value associated to the max enqueued request number in the asynchronous job executor thread pool is not displayed",
                output.contains(AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_MAX));

    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>no null value is returned for metric,</li>
     * <li>no error will occur.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nominal() throws MonitoringSubFunctionException {

        final long asyncExecutorThreadPoolActiveThreadCurrent = 123;
        final long asyncExecutorThreadPoolActiveThreadMax = 167;
        final long asyncExecutorThreadPoolIdleThreadCurrent = 2;
        final long asyncExecutorThreadPoolIdleThreadMax = 7;
        final long asyncExecutorThreadPoolEnqueuedRequestCurrent = 1;
        final long asyncExecutorThreadPoolEnqueuedRequestMax = 3;
        final long asyncExecutorThreadPoolMinSize = 5;
        final long asyncExecutorThreadPoolMaxSize = 25;
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = Long.valueOf(asyncExecutorThreadPoolActiveThreadCurrent);
        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = Long.valueOf(asyncExecutorThreadPoolActiveThreadMax);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = Long.valueOf(asyncExecutorThreadPoolIdleThreadCurrent);
        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = Long.valueOf(asyncExecutorThreadPoolIdleThreadMax);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = Long
                .valueOf(asyncExecutorThreadPoolEnqueuedRequestCurrent);
        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = Long.valueOf(asyncExecutorThreadPoolEnqueuedRequestMax);
        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = Long.valueOf(asyncExecutorThreadPoolMinSize);
        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = Long.valueOf(asyncExecutorThreadPoolMaxSize);
        this.subFunction.execute(new String[] { "-" + AsyncExecutorThreadPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        boolean isMissingAsyncExecutorThreadPoolActiveThreadCurrent = true;
        boolean isMissingAsyncExecutorThreadPoolActiveThreadMax = true;
        boolean isMissingAsyncExecutorThreadPoolIdleThreadCurrent = true;
        boolean isMissingAsyncExecutorThreadPoolIdleThreadMax = true;
        boolean isMissingAsyncExecutorThreadPoolEnqueuedRequestMax = true;
        boolean isMissingAsyncExecutorThreadPoolEnqueuedRequestCurrent = true;
        boolean isMissingAsyncExecutorThreadPoolMinSize = true;
        boolean isMissingAsyncExecutorThreadPoolMaxSize = true;
        final StringTokenizer strTokenizer = new StringTokenizer(output);
        while (strTokenizer.hasMoreElements()) {
            final StringTokenizer metricTokenizer = new StringTokenizer(strTokenizer.nextToken(), ":");
            final String metricLabel = metricTokenizer.nextToken();
            assertNotNull("missing metric label", metricLabel);
            final String metricValue = metricTokenizer.nextToken();
            assertNotNull("missing metric value", metricValue);
            if (AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_CURRENT.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current active threads of asynchronous job executor thread pool",
                        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT, Long.valueOf(metricValue));
                isMissingAsyncExecutorThreadPoolActiveThreadCurrent = false;
            } else if (AsyncExecutorThreadPool.HEADER_ACTIVE_THREAD_MAX.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max active threads of asynchronous job executor thread pool",
                        ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX, Long.valueOf(metricValue));
                isMissingAsyncExecutorThreadPoolActiveThreadMax = false;
            } else if (AsyncExecutorThreadPool.HEADER_IDLE_THREAD_MAX.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max idle threads of asynchronous job executor thread pool",
                        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX, Long.valueOf(metricValue));
                isMissingAsyncExecutorThreadPoolIdleThreadMax = false;
            } else if (AsyncExecutorThreadPool.HEADER_IDLE_THREAD_CURRENT.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current idle threads of asynchronous job executor thread pool",
                        ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT, Long.valueOf(metricValue));
                isMissingAsyncExecutorThreadPoolIdleThreadCurrent = false;
            } else if (AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_MAX.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max enqueued request in the asynchronous job executor thread pool",
                        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX, Long.valueOf(metricValue));
                isMissingAsyncExecutorThreadPoolEnqueuedRequestMax = false;
            } else if (AsyncExecutorThreadPool.HEADER_ENQUEUED_REQUEST_CURRENT.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current enqueued request into the asynchronous job executor thread pool",
                        ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT, Long.valueOf(metricValue));
                isMissingAsyncExecutorThreadPoolEnqueuedRequestCurrent = false;
            } else if (AsyncExecutorThreadPool.HEADER_MIN_SIZE.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the min size of the asynchronous job executor thread pool",
                        ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE, Long.valueOf(metricValue));
                isMissingAsyncExecutorThreadPoolMinSize = false;
            } else if (AsyncExecutorThreadPool.HEADER_MAX_SIZE.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max size of the asynchronous job executor thread pool",
                        ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE, Long.valueOf(metricValue));
                isMissingAsyncExecutorThreadPoolMaxSize = false;
            } else {
                fail("Unexpected metric");
            }
        }
        assertFalse(
                "The metric value associated to the current active threads of asynchronous job executor thread pool is missing",
                isMissingAsyncExecutorThreadPoolActiveThreadCurrent);
        assertFalse(
                "The metric value associated to the max active threads of asynchronous job executor thread pool is missing",
                isMissingAsyncExecutorThreadPoolActiveThreadMax);
        assertFalse(
                "The metric value associated to the current idle threads of asynchronous job executor thread pool is missing",
                isMissingAsyncExecutorThreadPoolIdleThreadCurrent);
        assertFalse(
                "The metric value associated to the max idle threads of asynchronous job executor thread pool is missing",
                isMissingAsyncExecutorThreadPoolIdleThreadMax);
        assertFalse(
                "The metric value associated to the current enqueued request into the asynchronous job executor thread pool is missing",
                isMissingAsyncExecutorThreadPoolEnqueuedRequestCurrent);
        assertFalse(
                "The metric value associated to the max enqueued request into the asynchronous job executor thread pool is missing",
                isMissingAsyncExecutorThreadPoolEnqueuedRequestMax);
        assertFalse("The metric value associated the max size of asynchronous job executor thread pool is missing",
                isMissingAsyncExecutorThreadPoolMaxSize);
        assertFalse("The metric value associated the min size of asynchronous job executor thread pool is missing",
                isMissingAsyncExecutorThreadPoolMinSize);
    }
}
