/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.se.flowable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE;

import java.util.List;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock;

/**
 * Unit tests of {@link DatabaseConnectionPool}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DatabaseConnectionPoolTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new DatabaseConnectionPool();
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void noOptionProvided() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedOption() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided does not exist.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the non existent component
     * </p>
     */
    @Test
    public void componentIdDoesNotExist() throws Exception {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] { "-" + DatabaseConnectionPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>error occurs getting asynchronous job executor thread pool values.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void errorGettingValues() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] { "-" + DatabaseConnectionPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max active connections of database connection pool,
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max active connections of database connection pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForMaxActiveConnections() throws MonitoringSubFunctionException {

        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX = null;
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + DatabaseConnectionPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse(
                "the metric value associated to the max active connections of database connection pool pool is displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current active connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_CURRENT));
        assertTrue("the metric value associated to the max active size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_ACTIVE_SIZE));
        assertTrue("the metric value associated to the max idle size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_IDLE_SIZE));
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current active connections of database connection
     * pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current active connections of database connection pool is not returned,
     * </li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForCurrentActiveConnections() throws MonitoringSubFunctionException {

        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT = null;
        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + DatabaseConnectionPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active connections of database connection pool pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_MAX));
        assertFalse(
                "the metric value associated to the current active connections of database connection pool is displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_CURRENT));
        assertTrue("the metric value associated to the max active size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_ACTIVE_SIZE));
        assertTrue("the metric value associated to the max idle size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_IDLE_SIZE));
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the current idle connections of database connection
     * pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the current idle connections of database connection pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForCurrentIdleConnections() throws MonitoringSubFunctionException {

        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT = null;
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + DatabaseConnectionPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active connections of database connection pool pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current active connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_MAX));
        assertFalse(
                "the metric value associated to the current idle connections of database connection pool is displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_CURRENT));
        assertTrue("the metric value associated to the max active size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_ACTIVE_SIZE));
        assertTrue("the metric value associated to the max idle size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_IDLE_SIZE));
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max idle connections of database connection pool,
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max idle connections of database connection pool is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForMaxIdleConnections() throws MonitoringSubFunctionException {

        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX = null;
        DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + DatabaseConnectionPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active connections of database connection pool pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current active connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_CURRENT));
        assertFalse("the metric value associated to the max idle connections of database connection pool is displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_CURRENT));
        assertTrue("the metric value associated to the max active size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_ACTIVE_SIZE));
        assertTrue("the metric value associated to the max idle size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_IDLE_SIZE));
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max size of active connections of database
     * connection pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max size of active connections of database connection pool is not
     * returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForMaxActiveSize() throws MonitoringSubFunctionException {

        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE = null;
        this.subFunction.execute(new String[] { "-" + DatabaseConnectionPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active connections of database connection pool pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current active connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_CURRENT));
        assertFalse("the metric value associated to the max active size of database connection pool is displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_ACTIVE_SIZE));
        assertTrue("the metric value associated to the max idle size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_IDLE_SIZE));
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the max size of idle connections of database connection
     * pool,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the max size of idle connections of database connection pool is not returned,
     * </li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nullValueForMaxIdleSize() throws MonitoringSubFunctionException {

        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX = Long.valueOf(123);
        DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE = null;
        DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE = Long.valueOf(123);
        this.subFunction.execute(new String[] { "-" + DatabaseConnectionPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "the metric value associated to the max active connections of database connection pool pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current active connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_CURRENT));
        assertTrue(
                "the metric value associated to the max idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_MAX));
        assertTrue(
                "the metric value associated to the current idle connections of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_CURRENT));
        assertTrue("the metric value associated to the max active size of database connection pool is not displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_ACTIVE_SIZE));
        assertFalse("the metric value associated to the max idle size of database connection pool is displayed",
                output.contains(DatabaseConnectionPool.HEADER_MAX_IDLE_SIZE));
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>no null value is returned for metric,</li>
     * <li>no error will occur.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void nominal() throws MonitoringSubFunctionException {

        final long databaseConnectionPoolActiveConnectionsCurrent = 123;
        final long databaseConnectionPoolActiveConnectionsMax = 167;
        final long databaseConnectionPoolIdleConnectionsCurrent = 2;
        final long databaseConnectionPoolIdleConnectionsMax = 7;
        final long databaseConnectionPoolMaxIdleSize = 5;
        final long databaseConnectionPoolMaxActiveSize = 25;
        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT = Long
                .valueOf(databaseConnectionPoolActiveConnectionsCurrent);
        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX = Long.valueOf(databaseConnectionPoolActiveConnectionsMax);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT = Long.valueOf(databaseConnectionPoolIdleConnectionsCurrent);
        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX = Long.valueOf(databaseConnectionPoolIdleConnectionsMax);
        DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE = Long.valueOf(databaseConnectionPoolMaxIdleSize);
        DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE = Long.valueOf(databaseConnectionPoolMaxActiveSize);
        this.subFunction.execute(new String[] { "-" + DatabaseConnectionPool.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        boolean isMissingDatabaseConnectionPoolActiveConnectionsCurrent = true;
        boolean isMissingDatabaseConnectionPoolActiveConnectionsMax = true;
        boolean isMissingDatabaseConnectionPoolIdleConnectionsCurrent = true;
        boolean isMissingDatabaseConnectionPoolIdleConnectionsMax = true;
        boolean isMissingDatabaseConnectionPoolMaxIdleSize = true;
        boolean isMissingDatabaseConnectionPoolMaxActiveSize = true;
        final StringTokenizer strTokenizer = new StringTokenizer(output);
        while (strTokenizer.hasMoreElements()) {
            final StringTokenizer metricTokenizer = new StringTokenizer(strTokenizer.nextToken(), ":");
            final String metricLabel = metricTokenizer.nextToken();
            assertNotNull("missing metric label", metricLabel);
            final String metricValue = metricTokenizer.nextToken();
            assertNotNull("missing metric value", metricValue);
            if (DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_CURRENT.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current active connections of database connection pool",
                        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT, Long.valueOf(metricValue));
                isMissingDatabaseConnectionPoolActiveConnectionsCurrent = false;
            } else if (DatabaseConnectionPool.HEADER_ACTIVE_CONNECTIONS_MAX.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max active connections of database connection pool pool",
                        DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX, Long.valueOf(metricValue));
                isMissingDatabaseConnectionPoolActiveConnectionsMax = false;
            } else if (DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_MAX.equals(metricLabel)) {
                assertEquals("invalid metric value associated to the max idle connections of database connection pool",
                        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX, Long.valueOf(metricValue));
                isMissingDatabaseConnectionPoolIdleConnectionsMax = false;
            } else if (DatabaseConnectionPool.HEADER_IDLE_CONNECTIONS_CURRENT.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the current idle connections of database connection pool",
                        DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT, Long.valueOf(metricValue));
                isMissingDatabaseConnectionPoolIdleConnectionsCurrent = false;
            } else if (DatabaseConnectionPool.HEADER_MAX_IDLE_SIZE.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max size of idle connections of database connection pool",
                        DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE, Long.valueOf(metricValue));
                isMissingDatabaseConnectionPoolMaxIdleSize = false;
            } else if (DatabaseConnectionPool.HEADER_MAX_ACTIVE_SIZE.equals(metricLabel)) {
                assertEquals(
                        "invalid metric value associated to the max size of active connections of database connection pool",
                        DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE, Long.valueOf(metricValue));
                isMissingDatabaseConnectionPoolMaxActiveSize = false;
            } else {
                fail("Unexpected metric");
            }
        }
        assertFalse(
                "The metric value associated to the current active connections of database connection pool is missing",
                isMissingDatabaseConnectionPoolActiveConnectionsCurrent);
        assertFalse(
                "The metric value associated to the max active connections of database connection pool pool is missing",
                isMissingDatabaseConnectionPoolActiveConnectionsMax);
        assertFalse(
                "The metric value associated to the current idle connections of database connection pool is missing",
                isMissingDatabaseConnectionPoolIdleConnectionsCurrent);
        assertFalse("The metric value associated to the max idle connections of database connection pool is missing",
                isMissingDatabaseConnectionPoolIdleConnectionsMax);
        assertFalse(
                "The metric value associated the max size of active connections of database connection pool is missing",
                isMissingDatabaseConnectionPoolMaxActiveSize);
        assertFalse(
                "The metric value associated the max size of idle connections of database connection pool is missing",
                isMissingDatabaseConnectionPoolMaxIdleSize);
    }
}
