/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.local;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.AbstractMonitoringSubFunctionTest;
import org.ow2.petals.jmx.api.mock.junit.monitoring.container.transporter.local.DeliveredMessagesMock;
import org.ow2.petals.jmx.api.mock.junit.monitoring.container.transporter.local.LocalTransporterMonitoringServiceClientMock;

/**
 * Unit tests of {@link DeliveredMessages}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DeliveredMessagesTest extends AbstractMonitoringSubFunctionTest {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new DeliveredMessages();
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>any option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void anyOptionProvided() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided,</li>
     * <li>no metrics are available (no call was done).</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs and metrics are returned with values '0'.
     * </p>
     */
    @Test
    public void noMetricsAvailable() throws MonitoringSubFunctionException {

        final LocalTransporterMonitoringServiceClientMock localTransporterMonitoringServiceClientMock = new LocalTransporterMonitoringServiceClientMock();
        this.jmxApi.registerLocalTransporterMonitoringServiceClient(localTransporterMonitoringServiceClientMock);

        // Execute test
        subFunction.execute(new String[] {});

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertMetrics(shell, 0l, 0l, 0l);
    }

    private static void assertMetrics(final StringStreamShell shell, final long expectedPending,
            final long expectedSucceeded, final long expectedError) {

        final String output = shell.getOutput();
        final StringTokenizer tokenizer = new StringTokenizer(output.trim(), " ");

        assertEquals(3, tokenizer.countTokens());
        assertMetric(tokenizer.nextToken(), DeliveredMessages.HEADER_PENDING, expectedPending);
        assertMetric(tokenizer.nextToken(), DeliveredMessages.HEADER_SUCCEEDED, expectedSucceeded);
        assertMetric(tokenizer.nextToken(), DeliveredMessages.HEADER_ERROR, expectedError);
    }

    private static void assertMetric(final String metricPart, String expectedHeader, long expectedValue) {

        final StringTokenizer tokenizer = new StringTokenizer(metricPart, ":");
        assertEquals(2, tokenizer.countTokens());
        assertEquals(expectedHeader, tokenizer.nextToken());
        assertEquals(expectedValue, Long.parseLong(tokenizer.nextToken()));
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided,</li>
     * <li>metrics are available.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs and the right metrics are returned.
     * </p>
     */
    @Test
    public void metricsAvailable() throws MonitoringSubFunctionException {

        final long expectedPending = 4;
        final long expectedSucceeded = 456;
        final long expectedError = 12;
        final LocalTransporterMonitoringServiceClientMock localTransporterMonitoringServiceClientMock = new LocalTransporterMonitoringServiceClientMock();
        localTransporterMonitoringServiceClientMock.setDeliveredMessagesMetrics(
                new DeliveredMessagesMock(expectedPending, expectedSucceeded, expectedError));
        this.jmxApi.registerLocalTransporterMonitoringServiceClient(localTransporterMonitoringServiceClientMock);

        // Execute test
        subFunction.execute(new String[] {});

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertMetrics(shell, expectedPending, expectedSucceeded, expectedError);
    }
}
