/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.OUTGOING_WS_REQUEST_RESPONSE_TIMES;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.OutgoingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.WsRequestExecutionStatus;
import org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock;

/**
 * Unit tests of {@link OutgoingWsRequestsResponseTimes}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class OutgoingWsRequestsResponseTimesTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new OutgoingWsRequestsResponseTimes();
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_02() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-"
                    + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option",
                    OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    e.getOption().getOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_03() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_04() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                            "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + OutgoingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                            "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_05() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + OutgoingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                    "--" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                    OutgoingWsRequestsResponseTimes.QUERY_ATTR_NAME });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_06() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + OutgoingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                    "-" + OutgoingWsRequestsResponseTimes.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_07() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option having a missing argument is not the expected one",
                    OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION, e.getOption()
                            .getLongOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with an invalid argument.
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void testExecute_Error_08() throws MonitoringSubFunctionException {

        final String invalidArg = "invalid-arg";
        try {
            this.subFunction.execute(new String[] {
                            "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                            invalidArg });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("The option having an invalid argument is not the expected one",
                    OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION, e.getOption()
                            .getLongOpt());
            assertEquals("The invalid value is not the expected one", invalidArg, e.getValue());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_09() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                    OutgoingWsRequestsResponseTimes.QUERY_ATTR_NAME,
                    "-" + OutgoingWsRequestsResponseTimes.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-t' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_10() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + OutgoingWsRequestsResponseTimes.TABLE_SHORT_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--list-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + OutgoingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_01() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "--" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + OutgoingWsRequestsResponseTimes.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_03() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + OutgoingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_04() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "--" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_05() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + OutgoingWsRequestsResponseTimes.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing
     * ws-request response times,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException {

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the outgoing ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing
     * ws-request response times,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException {

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the outgoing ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing
     * ws-request response times,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException, IOException {

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.TABLE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        final String header = brOutput.readLine();
        assertTrue("No header", header != null && !header.isEmpty());
        final String sepLine = brOutput.readLine();
        assertTrue("No line", sepLine != null && !sepLine.isEmpty());
        final String end = brOutput.readLine();
        assertNull("Extra character displayed: [" + end + "]", end);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-list-operations' is provided without argument,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException, IOException {

        final Long[] outgoingWsRequestResponseTimes1 = new Long[] { Long.valueOf(789),
                Long.valueOf(789), Long.valueOf(789), Long.valueOf(789), Long.valueOf(789),
                Long.valueOf(789) };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long[] outgoingWsRequestResponseTimes2 = new Long[] { Long.valueOf(10),
                Long.valueOf(10), Long.valueOf(10), Long.valueOf(10), Long.valueOf(10),
                Long.valueOf(10) };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES.put(new OutgoingWsRequestKey(outgoingWsRequestWsPath1,
                outgoingWsRequestWsOperation1, "clientIpAddress",
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimes1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES.put(new OutgoingWsRequestKey(outgoingWsRequestWsPath2,
                outgoingWsRequestWsOperation2, "clientIpAddress",
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimes2);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            boolean operationFound = false;
            for (final Map.Entry<OutgoingWsRequestKey, Long[]> entry : OUTGOING_WS_REQUEST_RESPONSE_TIMES
                    .entrySet()) {
                final OutgoingWsRequestKey key = entry.getKey();
                if (line.equals(key.getWsOperation() + "@" + key.getExternalWsUrl())) {
                    operationFound = true;
                    nbOperations++;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
        }
        assertEquals("unexpected operation number", OUTGOING_WS_REQUEST_RESPONSE_TIMES.size(),
                nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-query-operations' is provided with the argument "NAME",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException, IOException {

        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] { Long.valueOf(789),
                Long.valueOf(789), Long.valueOf(789), Long.valueOf(789), Long.valueOf(789),
                Long.valueOf(789) };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] { Long.valueOf(10),
                Long.valueOf(10), Long.valueOf(10), Long.valueOf(10), Long.valueOf(10),
                Long.valueOf(10) };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] { Long.valueOf(56),
                Long.valueOf(56), Long.valueOf(56), Long.valueOf(56), Long.valueOf(56),
                Long.valueOf(56) };
        final Long[] outgoingWsRequestResponseTimesFault3 = new Long[] { Long.valueOf(34),
                Long.valueOf(34), Long.valueOf(34), Long.valueOf(34), Long.valueOf(34),
                Long.valueOf(34) };
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] { Long.valueOf(9),
                Long.valueOf(9), Long.valueOf(9), Long.valueOf(9), Long.valueOf(9),
                Long.valueOf(9) };
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] { Long.valueOf(7),
                Long.valueOf(7), Long.valueOf(7), Long.valueOf(7), Long.valueOf(7),
                Long.valueOf(7) };

        OUTGOING_WS_REQUEST_RESPONSE_TIMES = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES.put(new OutgoingWsRequestKey(outgoingWsRequestWsPath1,
                outgoingWsRequestWsOperation1, null, WsRequestExecutionStatus.SUCCEEDED),
                outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES.put(new OutgoingWsRequestKey(outgoingWsRequestWsPath1,
                outgoingWsRequestWsOperation1, null, WsRequestExecutionStatus.FAULT),
                outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES.put(new OutgoingWsRequestKey(outgoingWsRequestWsPath1,
                outgoingWsRequestWsOperation1, null, WsRequestExecutionStatus.ERROR),
                outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES.put(new OutgoingWsRequestKey(outgoingWsRequestWsPath2,
                outgoingWsRequestWsOperation2, null, WsRequestExecutionStatus.SUCCEEDED),
                outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES.put(new OutgoingWsRequestKey(outgoingWsRequestWsPath2,
                outgoingWsRequestWsOperation2, null, WsRequestExecutionStatus.FAULT),
                outgoingWsRequestResponseTimesFault3);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES.put(new OutgoingWsRequestKey(outgoingWsRequestWsPath2,
                outgoingWsRequestWsOperation2, null, WsRequestExecutionStatus.ERROR),
                outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1, null);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2, null);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<OutgoingWsRequestKey, Long[]> entry : OUTGOING_WS_REQUEST_RESPONSE_TIMES
                    .entrySet()) {
                final OutgoingWsRequestKey key = entry.getKey();
                if (operationNameReturned.equals(key.getWsOperation() + "@"
                        + key.getExternalWsUrl())) {
                    operationFound = true;
                    assertEquals("Unexpected name value", operationNameReturned, value);
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMin() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_min);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesSucceeded2_min);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededAvg() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_avg);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesSucceeded2_avg);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMax() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_max);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesSucceeded2_max);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded10P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_10p);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesSucceeded2_10p);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_10P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded50P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_50p);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesSucceeded2_50p);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_50P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded90P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_90p);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesSucceeded2_90p);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_90P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMin() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_min);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesFault2_min);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MIN });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultAvg() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_avg);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesFault2_avg);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_AVG });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMax() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_max);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesFault2_max);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MAX });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault10P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_10p);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesFault2_10p);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_10P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute__Query_Fault50P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_50p);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesFault2_50p);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_50P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault90P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_90p);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesFault2_90p);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_90P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMin() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_min);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesError2_min);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MIN });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorAvg() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_avg);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesError2_avg);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_AVG });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMax() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_max);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesError2_max);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MAX });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error10P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_10p);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesError2_10p);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_10P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error50P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_50p);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesError2_50p);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_50P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error90P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_90p);
        expectedResults.put(outgoingWsRequestWsOperation2 + "@" + outgoingWsRequestWsPath2,
                outgoingWsRequestResponseTimesError2_90p);

        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_90P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiOutgoingResponseTimesOutput(output,
                OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_min);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_avg);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_max);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_10p);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_50p);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesSucceeded1_90p);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_min);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_avg);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_max);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_10p);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_50p);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesFault1_90p);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_min);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_avg);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_max);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_10p);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_50p);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(outgoingWsRequestWsOperation1 + "@" + outgoingWsRequestWsPath1,
                outgoingWsRequestResponseTimesError1_90p);

        final String queriedOperation = outgoingWsRequestWsOperation1 + "@"
                + outgoingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_FAULT_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long outgoingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long outgoingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long outgoingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long outgoingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long outgoingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long outgoingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] outgoingWsRequestResponseTimesSucceeded1 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded1_max,
                outgoingWsRequestResponseTimesSucceeded1_avg,
                outgoingWsRequestResponseTimesSucceeded1_min,
                outgoingWsRequestResponseTimesSucceeded1_10p,
                outgoingWsRequestResponseTimesSucceeded1_50p,
                outgoingWsRequestResponseTimesSucceeded1_90p };
        final String outgoingWsRequestWsPath1 = "wspath-1";
        final String outgoingWsRequestWsOperation1 = "wsOperation1";
        final Long outgoingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long outgoingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long outgoingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long outgoingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long outgoingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long outgoingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] outgoingWsRequestResponseTimesSucceeded2 = new Long[] {
                outgoingWsRequestResponseTimesSucceeded2_max,
                outgoingWsRequestResponseTimesSucceeded2_avg,
                outgoingWsRequestResponseTimesSucceeded2_min,
                outgoingWsRequestResponseTimesSucceeded2_10p,
                outgoingWsRequestResponseTimesSucceeded2_50p,
                outgoingWsRequestResponseTimesSucceeded2_90p };
        final String outgoingWsRequestWsPath2 = "wspath-2";
        final String outgoingWsRequestWsOperation2 = "wsOperation2";
        final Long outgoingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long outgoingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long outgoingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long outgoingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long outgoingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long outgoingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] outgoingWsRequestResponseTimesFault1 = new Long[] {
                outgoingWsRequestResponseTimesFault1_max, outgoingWsRequestResponseTimesFault1_avg,
                outgoingWsRequestResponseTimesFault1_min, outgoingWsRequestResponseTimesFault1_10p,
                outgoingWsRequestResponseTimesFault1_50p, outgoingWsRequestResponseTimesFault1_90p };
        final Long outgoingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long outgoingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long outgoingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long outgoingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long outgoingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long outgoingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] outgoingWsRequestResponseTimesFault2 = new Long[] {
                outgoingWsRequestResponseTimesFault2_max, outgoingWsRequestResponseTimesFault2_avg,
                outgoingWsRequestResponseTimesFault2_min, outgoingWsRequestResponseTimesFault2_10p,
                outgoingWsRequestResponseTimesFault2_50p, outgoingWsRequestResponseTimesFault2_90p };
        final Long outgoingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long outgoingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long outgoingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long outgoingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long outgoingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long outgoingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] outgoingWsRequestResponseTimesError1 = new Long[] {
                outgoingWsRequestResponseTimesError1_max, outgoingWsRequestResponseTimesError1_avg,
                outgoingWsRequestResponseTimesError1_min, outgoingWsRequestResponseTimesError1_10p,
                outgoingWsRequestResponseTimesError1_50p, outgoingWsRequestResponseTimesError1_90p };
        final Long outgoingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long outgoingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long outgoingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long outgoingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long outgoingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long outgoingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] outgoingWsRequestResponseTimesError2 = new Long[] {
                outgoingWsRequestResponseTimesError2_max, outgoingWsRequestResponseTimesError2_avg,
                outgoingWsRequestResponseTimesError2_min, outgoingWsRequestResponseTimesError2_10p,
                outgoingWsRequestResponseTimesError2_50p, outgoingWsRequestResponseTimesError2_90p };

        // Outgoing ws request response times grouped by service, operation and
        // exec status is required.
        OUTGOING_WS_REQUEST_RESPONSE_TIMES = null;
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault1);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath1, outgoingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError1);

        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), outgoingWsRequestResponseTimesSucceeded2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), outgoingWsRequestResponseTimesFault2);
        OUTGOING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new OutgoingWsRequestKey(
                outgoingWsRequestWsPath2, outgoingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), outgoingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + OutgoingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + OutgoingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                OutgoingWsRequestsResponseTimes.QUERY_ATTR_ERROR_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }
}
