/**
 * Copyright (c) 2013-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES;
import static org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock.SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationKey;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationStatus;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;

/**
 * Unit tests of {@link ServiceProviderInvocationsResponseTimes}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ServiceProviderInvocationsResponseTimesTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new ServiceProviderInvocationsResponseTimes();
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_02() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(
                    new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option", ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    e.getOption().getOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>no other option is provided.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_03() throws MonitoringSubFunctionException {

        try {
            subFunction
                    .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_04() throws MonitoringSubFunctionException {

        try {
            subFunction
                    .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvocationsResponseTimes.LIST_OPERATIONS_LONG_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_05() throws MonitoringSubFunctionException {

        try {
            subFunction
                    .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvocationsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                            "--" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                            ServiceProviderInvocationsResponseTimes.QUERY_ATTR_NAME });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_06() throws MonitoringSubFunctionException {

        try {
            subFunction
                    .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvocationsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                            "-" + ServiceProviderInvocationsResponseTimes.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_07() throws MonitoringSubFunctionException {

        try {
            subFunction
                    .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option having a missing argument is not the expected one",
                    ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION, e.getOption().getLongOpt());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with an invalid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void testExecute_Error_08() throws MonitoringSubFunctionException {

        final String invalidArg = "invalid-arg";
        try {
            subFunction
                    .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION, invalidArg });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("The option having an invalid argument is not the expected one",
                    ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION, e.getOption().getLongOpt());
            assertEquals("The invalid value is not the expected one", invalidArg, e.getValue());
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_09() throws MonitoringSubFunctionException {

        try {
            subFunction
                    .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                            ServiceProviderInvocationsResponseTimes.QUERY_ATTR_NAME,
                            "-" + ServiceProviderInvocationsResponseTimes.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against argument management, where:
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-t' is provided with an argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_10() throws MonitoringSubFunctionException {

        try {
            subFunction
                    .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                            ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "-" + ServiceProviderInvocationsResponseTimes.TABLE_SHORT_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--list-operations' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                        "-" + ServiceProviderInvocationsResponseTimes.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_01() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                        "--" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                        "-" + ServiceProviderInvocationsResponseTimes.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_03() throws MonitoringSubFunctionException {

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                        "-" + ServiceProviderInvocationsResponseTimes.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_04() throws MonitoringSubFunctionException {

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                        "--" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_NAME });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_05() throws MonitoringSubFunctionException {

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                        "-" + ServiceProviderInvocationsResponseTimes.TABLE_SHORT_OPTION });
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing ws-request response times,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException {

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = null;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed", output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing ws-request response times,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException {

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = null;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed", output.isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing ws-request response times,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException, IOException {

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = null;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.TABLE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        final String header = brOutput.readLine();
        assertTrue("No header", header != null && !header.isEmpty());
        final String sepLine = brOutput.readLine();
        assertTrue("No line", sepLine != null && !sepLine.isEmpty());
        final String end = brOutput.readLine();
        assertNull("Extra character displayed: [" + end + "]", end);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-list-operations' is provided without argument,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException, IOException {

        final Long[] serviceProviderInvocationsResponseTimes1 = new Long[] { Long.valueOf(789), Long.valueOf(789),
                Long.valueOf(789), Long.valueOf(789), Long.valueOf(789), Long.valueOf(789) };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long[] serviceProviderInvocationResponseTimes2 = new Long[] { Long.valueOf(10), Long.valueOf(10),
                Long.valueOf(10), Long.valueOf(10), Long.valueOf(10), Long.valueOf(10) };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, "clientIpAddress",
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationsResponseTimes1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, "clientIpAddress",
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimes2);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvocationKey, Long[]> entry : SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                    .entrySet()) {
                final ServiceProviderInvocationKey key = entry.getKey();
                if (line.equals(key.getOperation() + "@" + key.getServiceName() + "@" + key.getInterfaceName())) {
                    operationFound = true;
                    nbOperations++;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
        }
        assertEquals("unexpected operation number", SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES.size(), nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-query-operations' is provided with the argument "NAME",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException, IOException {

        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] { Long.valueOf(789),
                Long.valueOf(789), Long.valueOf(789), Long.valueOf(789), Long.valueOf(789), Long.valueOf(789) };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] { Long.valueOf(10), Long.valueOf(10),
                Long.valueOf(10), Long.valueOf(10), Long.valueOf(10), Long.valueOf(10) };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] { Long.valueOf(56), Long.valueOf(56),
                Long.valueOf(56), Long.valueOf(56), Long.valueOf(56), Long.valueOf(56) };
        final Long[] serviceProviderInvocationResponseTimesFault3 = new Long[] { Long.valueOf(34), Long.valueOf(34),
                Long.valueOf(34), Long.valueOf(34), Long.valueOf(34), Long.valueOf(34) };
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] { Long.valueOf(9), Long.valueOf(9),
                Long.valueOf(9), Long.valueOf(9), Long.valueOf(9), Long.valueOf(9) };
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] { Long.valueOf(7), Long.valueOf(7),
                Long.valueOf(7), Long.valueOf(7), Long.valueOf(7), Long.valueOf(7) };

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault3);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1, null);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2, null);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<ServiceProviderInvocationKey, Long[]> entry : SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES
                    .entrySet()) {
                final ServiceProviderInvocationKey key = entry.getKey();
                if (operationNameReturned
                        .equals(key.getOperation() + "@" + key.getServiceName() + "@" + key.getInterfaceName())) {
                    operationFound = true;
                    assertEquals("Unexpected name value", operationNameReturned, value);
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMin()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_min);
        expectedResults.put(
                serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                        + serviceProviderInvocationInterfaceName2,
                serviceProviderInvocationResponseTimesSucceeded2_min);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededAvg()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_avg);
        expectedResults.put(
                serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                        + serviceProviderInvocationInterfaceName2,
                serviceProviderInvocationResponseTimesSucceeded2_avg);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMax()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_max);
        expectedResults.put(
                serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                        + serviceProviderInvocationInterfaceName2,
                serviceProviderInvocationResponseTimesSucceeded2_max);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded10P()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_10p);
        expectedResults.put(
                serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                        + serviceProviderInvocationInterfaceName2,
                serviceProviderInvocationResponseTimesSucceeded2_10p);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_10P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded50P()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf1-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf1-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_50p);
        expectedResults.put(
                serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                        + serviceProviderInvocationInterfaceName2,
                serviceProviderInvocationResponseTimesSucceeded2_50p);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_50P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded90P()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf1-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf1-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_90p);
        expectedResults.put(
                serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                        + serviceProviderInvocationInterfaceName2,
                serviceProviderInvocationResponseTimesSucceeded2_90p);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_90P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMin()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_min);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesFault2_min);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_MIN });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultAvg()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_avg);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesFault2_avg);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_AVG });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMax()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_max);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesFault2_max);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_MAX });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault10P()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_10p);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesFault2_10p);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_10P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute__Query_Fault50P()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_50p);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesFault2_50p);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_50P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault90P()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_90p);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesFault2_90p);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_90P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMin()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_min);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesError2_min);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_MIN });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorAvg()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_avg);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesError2_avg);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_AVG });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMax()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_max);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesError2_max);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_MAX });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error10P()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_10p);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesError2_10p);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_10P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error50P()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_50p);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesError2_50p);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_50P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error90P()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_90p);
        expectedResults.put(serviceProviderInvocationOperation2 + "@" + serviceProviderInvocationServiceName2 + "@"
                + serviceProviderInvocationInterfaceName2, serviceProviderInvocationResponseTimesError2_90p);

        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_90P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiServiceProviderInvocationResponseTimesOutput(output,
                SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS, expectedResults);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_min);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_avg);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_max);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_10p);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_50p);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(
                serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                        + serviceProviderInvocationInterfaceName1,
                serviceProviderInvocationResponseTimesSucceeded1_90p);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_min);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_avg);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_max);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_10p);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_50p);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesFault1_90p);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_min);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_avg);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_max);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_10p);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_50p);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, IOException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(serviceProviderInvocationOperation1 + "@" + serviceProviderInvocationServiceName1 + "@"
                + serviceProviderInvocationInterfaceName1, serviceProviderInvocationResponseTimesError1_90p);

        final String queriedOperation = serviceProviderInvocationOperation1 + "@"
                + serviceProviderInvocationServiceName1 + "@" + serviceProviderInvocationInterfaceName1;
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults, queriedOperation);
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_SucceededMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Succeeded90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_SUCCEEDED_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_FaultMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Fault90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_FAULT_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_ErrorMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * Test the method 'execute()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecute_Query_Error90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long serviceProviderInvocationResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long serviceProviderInvocationResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long serviceProviderInvocationResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long serviceProviderInvocationResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long serviceProviderInvocationResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long serviceProviderInvocationResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] serviceProviderInvocationResponseTimesSucceeded1 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded1_max,
                serviceProviderInvocationResponseTimesSucceeded1_avg,
                serviceProviderInvocationResponseTimesSucceeded1_min,
                serviceProviderInvocationResponseTimesSucceeded1_10p,
                serviceProviderInvocationResponseTimesSucceeded1_50p,
                serviceProviderInvocationResponseTimesSucceeded1_90p };
        final String serviceProviderInvocationInterfaceName1 = "itf-1";
        final String serviceProviderInvocationServiceName1 = "svc-1";
        final String serviceProviderInvocationOperation1 = "operation1";
        final Long serviceProviderInvocationResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long serviceProviderInvocationResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long serviceProviderInvocationResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long serviceProviderInvocationResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long serviceProviderInvocationResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long serviceProviderInvocationResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] serviceProviderInvocationResponseTimesSucceeded2 = new Long[] {
                serviceProviderInvocationResponseTimesSucceeded2_max,
                serviceProviderInvocationResponseTimesSucceeded2_avg,
                serviceProviderInvocationResponseTimesSucceeded2_min,
                serviceProviderInvocationResponseTimesSucceeded2_10p,
                serviceProviderInvocationResponseTimesSucceeded2_50p,
                serviceProviderInvocationResponseTimesSucceeded2_90p };
        final String serviceProviderInvocationInterfaceName2 = "itf-2";
        final String serviceProviderInvocationServiceName2 = "svc-2";
        final String serviceProviderInvocationOperation2 = "operation2";
        final Long serviceProviderInvocationResponseTimesFault1_min = Long.valueOf(32);
        final Long serviceProviderInvocationResponseTimesFault1_avg = Long.valueOf(35);
        final Long serviceProviderInvocationResponseTimesFault1_max = Long.valueOf(41);
        final Long serviceProviderInvocationResponseTimesFault1_10p = Long.valueOf(33);
        final Long serviceProviderInvocationResponseTimesFault1_50p = Long.valueOf(34);
        final Long serviceProviderInvocationResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] serviceProviderInvocationResponseTimesFault1 = new Long[] {
                serviceProviderInvocationResponseTimesFault1_max, serviceProviderInvocationResponseTimesFault1_avg,
                serviceProviderInvocationResponseTimesFault1_min, serviceProviderInvocationResponseTimesFault1_10p,
                serviceProviderInvocationResponseTimesFault1_50p, serviceProviderInvocationResponseTimesFault1_90p };
        final Long serviceProviderInvocationResponseTimesFault2_min = Long.valueOf(47);
        final Long serviceProviderInvocationResponseTimesFault2_avg = Long.valueOf(50);
        final Long serviceProviderInvocationResponseTimesFault2_max = Long.valueOf(56);
        final Long serviceProviderInvocationResponseTimesFault2_10p = Long.valueOf(48);
        final Long serviceProviderInvocationResponseTimesFault2_50p = Long.valueOf(49);
        final Long serviceProviderInvocationResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] serviceProviderInvocationResponseTimesFault2 = new Long[] {
                serviceProviderInvocationResponseTimesFault2_max, serviceProviderInvocationResponseTimesFault2_avg,
                serviceProviderInvocationResponseTimesFault2_min, serviceProviderInvocationResponseTimesFault2_10p,
                serviceProviderInvocationResponseTimesFault2_50p, serviceProviderInvocationResponseTimesFault2_90p };
        final Long serviceProviderInvocationResponseTimesError1_min = Long.valueOf(62);
        final Long serviceProviderInvocationResponseTimesError1_avg = Long.valueOf(65);
        final Long serviceProviderInvocationResponseTimesError1_max = Long.valueOf(71);
        final Long serviceProviderInvocationResponseTimesError1_10p = Long.valueOf(63);
        final Long serviceProviderInvocationResponseTimesError1_50p = Long.valueOf(64);
        final Long serviceProviderInvocationResponseTimesError1_90p = Long.valueOf(69);
        final Long[] serviceProviderInvocationResponseTimesError1 = new Long[] {
                serviceProviderInvocationResponseTimesError1_max, serviceProviderInvocationResponseTimesError1_avg,
                serviceProviderInvocationResponseTimesError1_min, serviceProviderInvocationResponseTimesError1_10p,
                serviceProviderInvocationResponseTimesError1_50p, serviceProviderInvocationResponseTimesError1_90p };
        final Long serviceProviderInvocationResponseTimesError2_min = Long.valueOf(77);
        final Long serviceProviderInvocationResponseTimesError2_avg = Long.valueOf(80);
        final Long serviceProviderInvocationResponseTimesError2_max = Long.valueOf(86);
        final Long serviceProviderInvocationResponseTimesError2_10p = Long.valueOf(78);
        final Long serviceProviderInvocationResponseTimesError2_50p = Long.valueOf(79);
        final Long serviceProviderInvocationResponseTimesError2_90p = Long.valueOf(84);
        final Long[] serviceProviderInvocationResponseTimesError2 = new Long[] {
                serviceProviderInvocationResponseTimesError2_max, serviceProviderInvocationResponseTimesError2_avg,
                serviceProviderInvocationResponseTimesError2_min, serviceProviderInvocationResponseTimesError2_10p,
                serviceProviderInvocationResponseTimesError2_50p, serviceProviderInvocationResponseTimesError2_90p };

        // Service provider invocation response times grouped by interface,
        // service, operation and
        // exec status is required.
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES = null;
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS = new HashMap<>();
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault1);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName1,
                        serviceProviderInvocationServiceName1, serviceProviderInvocationOperation1, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError1);

        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.SUCCEEDED), serviceProviderInvocationResponseTimesSucceeded2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.FAULT), serviceProviderInvocationResponseTimesFault2);
        SERVICE_PROVIDER_INVOCATIONS_RESPONSE_TIMES_GRP_BY_ITF_SVC_OP_AND_STATUS
                .put(new ServiceProviderInvocationKey(serviceProviderInvocationInterfaceName2,
                        serviceProviderInvocationServiceName2, serviceProviderInvocationOperation2, null,
                        ServiceProviderInvocationStatus.ERROR), serviceProviderInvocationResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        subFunction
                .execute(new String[] { "-" + ServiceProviderInvocationsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                        ComponentMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + ServiceProviderInvocationsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                        ServiceProviderInvocationsResponseTimes.QUERY_ATTR_ERROR_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }
}
